@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 """
 /***************************************************************************
 Name                 : Virtual layers plugin for DB Manager
@@ -24,7 +22,15 @@ from qgis.PyQt.QtCore import QUrl, QTemporaryFile
 from ..connector import DBConnector
 from ..plugin import Table
 
-from qgis.core import QGis, QgsDataSourceURI, QgsVirtualLayerDefinition, QgsMapLayerRegistry, QgsMapLayer, QgsVectorLayer, QgsCoordinateReferenceSystem
+from qgis.core import (
+    QgsDataSourceUri,
+    QgsVirtualLayerDefinition,
+    QgsProject,
+    QgsMapLayerType,
+    QgsVectorLayer,
+    QgsCoordinateReferenceSystem,
+    QgsWkbTypes,
+)
 
 import sqlite3
 
@@ -37,8 +43,9 @@ class sqlite3_connection:
     def __enter__(self):
         return self.conn
 
-    def __exit__(self, type, value, traceback):
+    def __exit__(self, ex_type, value, traceback):
         self.conn.close()
+        return ex_type is None
 
 
 def getQueryGeometryName(sqlite_file):
@@ -46,7 +53,7 @@ def getQueryGeometryName(sqlite_file):
     with sqlite3_connection(sqlite_file) as conn:
         c = conn.cursor()
         for r in c.execute("SELECT url FROM _meta"):
-            d = QgsVirtualLayerDefinition.fromUrl(QUrl.fromEncoded(r[0]))
+            d = QgsVirtualLayerDefinition.fromUrl(QUrl(r[0]))
             if d.hasDefinedGeometry():
                 return d.geometryField()
         return None
@@ -96,17 +103,18 @@ class VLayerRegistry:
         lid = self.layers.get(l)
         if lid is None:
             return lid
-        # the instance can refer to a layer in map previe and not in qgis general canvas
-        if lid not in QgsMapLayerRegistry.instance().mapLayers().keys():
+        if lid not in QgsProject.instance().mapLayers().keys():
             self.layers.pop(l)
             return None
-        return QgsMapLayerRegistry.instance().mapLayer(lid)
+        return QgsProject.instance().mapLayer(lid)
 
 
 class VLayerConnector(DBConnector):
 
     def __init__(self, uri):
-        pass
+        super().__init__(uri)
+
+        self.mapSridToName = {}
 
     def _execute(self, cursor, sql):
         # This is only used to get list of fields
@@ -117,10 +125,12 @@ class VLayerConnector(DBConnector):
 
             def close(self):
                 pass
+
         return DummyCursor(sql)
 
     def _get_cursor(self, name=None):
-        print("_get_cursor_", name)
+        # fix_print_with_import
+        print(("_get_cursor_", name))
 
     def _get_cursor_columns(self, c):
         tf = QTemporaryFile()
@@ -128,19 +138,21 @@ class VLayerConnector(DBConnector):
         tmp = tf.fileName()
         tf.close()
 
-        q = QUrl.toPercentEncoding(c.sql)
-        p = QgsVectorLayer("%s?query=%s" % (QUrl.fromLocalFile(tmp).toString(), q), "vv", "virtual")
+        df = QgsVirtualLayerDefinition()
+        df.setFilePath(tmp)
+        df.setQuery(c.sql)
+        p = QgsVectorLayer(df.toString(), "vv", "virtual")
         if not p.isValid():
             return []
         f = [f.name() for f in p.fields()]
-        if p.geometryType() != QGis.WKBNoGeometry:
+        if p.geometryType() != QgsWkbTypes.GeometryType.NullGeometry:
             gn = getQueryGeometryName(tmp)
             if gn:
                 f += [gn]
         return f
 
     def uri(self):
-        return QgsDataSourceURI("qgis")
+        return QgsDataSourceUri("qgis")
 
     def getInfo(self):
         return "info"
@@ -162,37 +174,46 @@ class VLayerConnector(DBConnector):
 
     def fieldTypes(self):
         return [
-            "integer", "bigint", "smallint",  # integers
-            "real", "double", "float", "numeric",  # floats
-            "varchar", "varchar(255)", "character(20)", "text",  # strings
-            "date", "datetime"  # date/time
+            "integer",
+            "bigint",
+            "smallint",  # integers
+            "real",
+            "double",
+            "float",
+            "numeric",  # floats
+            "varchar",
+            "varchar(255)",
+            "character(20)",
+            "text",  # strings
+            "date",
+            "datetime",  # date/time
         ]
 
     def getSchemas(self):
         return None
 
     def getTables(self, schema=None, add_sys_tables=False):
-        """ get list of tables """
+        """get list of tables"""
         return self.getVectorTables()
 
     def getVectorTables(self, schema=None):
-        """ get list of table with a geometry column
-                it returns:
-                        name (table name)
-                        is_system_table
-                        type = 'view' (is a view?)
-                        geometry_column:
-                                f_table_name (the table name in geometry_columns may be in a wrong case, use this to load the layer)
-                                f_geometry_column
-                                type
-                                coord_dimension
-                                srid
+        """get list of table with a geometry column
+        it returns:
+                name (table name)
+                is_system_table
+                type = 'view' (is a view?)
+                geometry_column:
+                        f_table_name (the table name in geometry_columns may be in a wrong case, use this to load the layer)
+                        f_geometry_column
+                        type
+                        coord_dimension
+                        srid
         """
         reg = VLayerRegistry.instance()
         VLayerRegistry.instance().reset()
         lst = []
-        for _, l in list(QgsMapLayerRegistry.instance().mapLayers().items()):
-            if l.type() == QgsMapLayer.VectorLayer:
+        for _, l in QgsProject.instance().mapLayers().items():
+            if l.type() == QgsMapLayerType.VectorLayer:
 
                 lname = l.name()
                 # if there is already a layer with this name, use the layer id
@@ -203,45 +224,34 @@ class VLayerConnector(DBConnector):
 
                 geomType = None
                 dim = None
-                g = l.dataProvider().geometryType()
-                if g == QGis.WKBPoint:
-                    geomType = 'POINT'
-                    dim = 'XY'
-                elif g == QGis.WKBLineString:
-                    geomType = 'LINESTRING'
-                    dim = 'XY'
-                elif g == QGis.WKBPolygon:
-                    geomType = 'POLYGON'
-                    dim = 'XY'
-                elif g == QGis.WKBMultiPoint:
-                    geomType = 'MULTIPOINT'
-                    dim = 'XY'
-                elif g == QGis.WKBMultiLineString:
-                    geomType = 'MULTILINESTRING'
-                    dim = 'XY'
-                elif g == QGis.WKBMultiPolygon:
-                    geomType = 'MULTIPOLYGON'
-                    dim = 'XY'
-                elif g == QGis.WKBPoint25D:
-                    geomType = 'POINT'
-                    dim = 'XYZ'
-                elif g == QGis.WKBLineString25D:
-                    geomType = 'LINESTRING'
-                    dim = 'XYZ'
-                elif g == QGis.WKBPolygon25D:
-                    geomType = 'POLYGON'
-                    dim = 'XYZ'
-                elif g == QGis.WKBMultiPoint25D:
-                    geomType = 'MULTIPOINT'
-                    dim = 'XYZ'
-                elif g == QGis.WKBMultiLineString25D:
-                    geomType = 'MULTILINESTRING'
-                    dim = 'XYZ'
-                elif g == QGis.WKBMultiPolygon25D:
-                    geomType = 'MULTIPOLYGON'
-                    dim = 'XYZ'
-                lst.append(
-                    (Table.VectorType, lname, False, False, l.id(), 'geometry', geomType, dim, l.crs().postgisSrid()))
+                if l.isSpatial():
+                    g = l.dataProvider().wkbType()
+                    g_flat = QgsWkbTypes.flatType(g)
+                    geomType = QgsWkbTypes.displayString(g_flat).upper()
+                    if geomType:
+                        dim = "XY"
+                        if QgsWkbTypes.hasZ(g):
+                            dim += "Z"
+                        if QgsWkbTypes.hasM(g):
+                            dim += "M"
+                    srid = l.crs().postgisSrid()
+                    if srid not in self.mapSridToName:
+                        self.mapSridToName[srid] = l.crs().description()
+                    lst.append(
+                        (
+                            Table.VectorType,
+                            lname,
+                            False,
+                            False,
+                            l.id(),
+                            "geometry",
+                            geomType,
+                            dim,
+                            srid,
+                        )
+                    )
+                else:
+                    lst.append((Table.TableType, lname, False, False))
         return lst
 
     def getRasterTables(self, schema=None):
@@ -255,16 +265,19 @@ class VLayerConnector(DBConnector):
         return l.featureCount()
 
     def getTableFields(self, table):
-        """ return list of columns in table """
+        """return list of columns in table"""
         t = table[1]
         l = VLayerRegistry.instance().getLayer(t)
         if not l or not l.isValid():
             return []
         # id, name, type, nonnull, default, pk
         n = l.dataProvider().fields().size()
-        f = [(i, f.name(), f.typeName(), False, None, False)
-             for i, f in enumerate(l.dataProvider().fields())]
-        f += [(n, "geometry", "geometry", False, None, False)]
+        f = [
+            (i, f.name(), f.typeName(), False, None, False)
+            for i, f in enumerate(l.dataProvider().fields())
+        ]
+        if l.isSpatial():
+            f += [(n, "geometry", "geometry", False, None, False)]
         return f
 
     def getTableIndexes(self, table):
@@ -282,7 +295,7 @@ class VLayerConnector(DBConnector):
     def getTableExtent(self, table, geom):
         is_id, t = table
         if is_id:
-            l = QgsMapLayerRegistry.instance().mapLayer(t)
+            l = QgsProject.instance().mapLayer(t)
         else:
             l = VLayerRegistry.instance().getLayer(t)
         if not l or not l.isValid():
@@ -295,8 +308,7 @@ class VLayerConnector(DBConnector):
         print("**unimplemented** getViewDefinition")
 
     def getSpatialRefInfo(self, srid):
-        crs = QgsCoordinateReferenceSystem(srid)
-        return crs.description()
+        return self.mapSridToName.get(srid, "")
 
     def isVectorTable(self, table):
         return True
@@ -347,7 +359,16 @@ class VLayerConnector(DBConnector):
     def deleteTableColumn(self, table, column):
         print("**unimplemented** deleteTableColumn")
 
-    def updateTableColumn(self, table, column, new_name, new_data_type=None, new_not_null=None, new_default=None):
+    def updateTableColumn(
+        self,
+        table,
+        column,
+        new_name,
+        new_data_type=None,
+        new_not_null=None,
+        new_default=None,
+        comment=None,
+    ):
         print("**unimplemented** updateTableColumn")
 
     def renameTableColumn(self, table, column, new_name):
@@ -370,7 +391,9 @@ class VLayerConnector(DBConnector):
         print("**unimplemented** isGeometryColumn")
         return False
 
-    def addGeometryColumn(self, table, geom_column='geometry', geom_type='POINT', srid=-1, dim=2):
+    def addGeometryColumn(
+        self, table, geom_column="geometry", geom_type="POINT", srid=-1, dim=2
+    ):
         print("**unimplemented** addGeometryColumn")
         return False
 
@@ -398,15 +421,15 @@ class VLayerConnector(DBConnector):
         print("**unimplemented** deleteTableIndex")
         return False
 
-    def createSpatialIndex(self, table, geom_column='geometry'):
+    def createSpatialIndex(self, table, geom_column="geometry"):
         print("**unimplemented** createSpatialIndex")
         return False
 
-    def deleteSpatialIndex(self, table, geom_column='geometry'):
+    def deleteSpatialIndex(self, table, geom_column="geometry"):
         print("**unimplemented** deleteSpatialIndex")
         return False
 
-    def hasSpatialIndex(self, table, geom_column='geometry'):
+    def hasSpatialIndex(self, table, geom_column="geometry"):
         print("**unimplemented** hasSpatialIndex")
         return False
 
@@ -420,6 +443,7 @@ class VLayerConnector(DBConnector):
 
     def getSqlDictionary(self):
         from .sql_dictionary import getSqlDictionary
+
         sql_dict = getSqlDictionary()
 
         items = []
