@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 """
 /***************************************************************************
 Name                 : DB Manager
@@ -22,13 +20,23 @@ The content of this file is based on
  ***************************************************************************/
 """
 
-from qgis.PyQt.QtCore import Qt, QSettings, QFileInfo
+from qgis.PyQt import uic
+from qgis.PyQt.QtCore import Qt, QFileInfo
 from qgis.PyQt.QtWidgets import QDialog, QFileDialog, QMessageBox, QApplication
 from qgis.PyQt.QtGui import QCursor
 
-from qgis.core import QgsVectorFileWriter, QgsVectorDataProvider, QgsCoordinateReferenceSystem, QgsVectorLayerImport
+from qgis.core import (
+    QgsVectorFileWriter,
+    QgsVectorDataProvider,
+    QgsCoordinateReferenceSystem,
+    QgsVectorLayerExporter,
+    QgsSettings,
+)
+from qgis.utils import OverrideCursor
 
-from .ui.ui_DlgExportVector import Ui_DbManagerDlgExportVector as Ui_Dialog
+from .gui_utils import GuiUtils
+
+Ui_Dialog, _ = uic.loadUiType(GuiUtils.get_ui_file_path("DlgExportVector.ui"))
 
 
 class DlgExportVector(QDialog, Ui_Dialog):
@@ -40,8 +48,8 @@ class DlgExportVector(QDialog, Ui_Dialog):
         self.setupUi(self)
 
         vectorFilterName = "lastVectorFileFilter"  # "lastRasterFileFilter"
-        self.lastUsedVectorFilterSettingsKey = u"/UI/{0}".format(vectorFilterName)
-        self.lastUsedVectorDirSettingsKey = u"/UI/{0}Dir".format(vectorFilterName)
+        self.lastUsedVectorFilterSettingsKey = f"/UI/{vectorFilterName}"
+        self.lastUsedVectorDirSettingsKey = f"/UI/{vectorFilterName}Dir"
 
         # update UI
         self.setupWorkingMode()
@@ -59,36 +67,39 @@ class DlgExportVector(QDialog, Ui_Dialog):
         self.checkSupports()
 
     def checkSupports(self):
-        """ update options available for the current input layer """
+        """update options available for the current input layer"""
         allowSpatial = self.db.connector.hasSpatialSupport()
-        hasGeomType = self.inLayer and self.inLayer.hasGeometryType()
+        hasGeomType = self.inLayer and self.inLayer.isSpatial()
         self.chkSourceSrid.setEnabled(allowSpatial and hasGeomType)
         self.chkTargetSrid.setEnabled(allowSpatial and hasGeomType)
         # self.chkSpatialIndex.setEnabled(allowSpatial and hasGeomType)
 
     def chooseOutputFile(self):
         # get last used dir
-        settings = QSettings()
+        settings = QgsSettings()
         lastUsedDir = settings.value(self.lastUsedVectorDirSettingsKey, ".")
 
         # get selected filter
-        selectedFilter = self.cboFileFormat.itemData(self.cboFileFormat.currentIndex())
+        selected_driver = self.cboFileFormat.currentData()
+        selected_filter = QgsVectorFileWriter.filterForDriver(selected_driver)
 
         # ask for a filename
-        filename = QFileDialog.getSaveFileName(self, self.tr("Choose where to save the file"), lastUsedDir,
-                                               selectedFilter)
+        filename, filter = QFileDialog.getSaveFileName(
+            self, self.tr("Choose where to save the file"), lastUsedDir, selected_filter
+        )
         if filename == "":
             return
 
-        filterString = QgsVectorFileWriter.filterForDriver(selectedFilter)
-        ext = filterString[filterString.find('.'):]
-        ext = ext[:ext.find(' ')]
+        ext = selected_filter[selected_filter.find(".") :]
+        ext = ext[: ext.find(" ")]
 
         if not filename.lower().endswith(ext):
             filename += ext
 
         # store the last used dir
-        settings.setValue(self.lastUsedVectorDirSettingsKey, QFileInfo(filename).filePath())
+        settings.setValue(
+            self.lastUsedVectorDirSettingsKey, QFileInfo(filename).filePath()
+        )
 
         self.editOutputFile.setText(filename)
 
@@ -106,12 +117,12 @@ class DlgExportVector(QDialog, Ui_Dialog):
 
     def populateFileFilters(self):
         # populate the combo with supported vector file formats
-        for name, filt in QgsVectorFileWriter.ogrDriverList().items():
-            self.cboFileFormat.addItem(name, filt)
+        for driver in QgsVectorFileWriter.ogrDriverList():
+            self.cboFileFormat.addItem(driver.longName, driver.driverName)
 
         # set the last used filter
-        settings = QSettings()
-        filt = settings.value(self.lastUsedVectorFilterSettingsKey, "ESRI Shapefile")
+        settings = QgsSettings()
+        filt = settings.value(self.lastUsedVectorFilterSettingsKey, "GPKG")
 
         idx = self.cboFileFormat.findText(filt)
         if idx < 0:
@@ -121,87 +132,90 @@ class DlgExportVector(QDialog, Ui_Dialog):
     def accept(self):
         # sanity checks
         if self.editOutputFile.text() == "":
-            QMessageBox.information(self, self.tr("Export to file"), self.tr("Output file name is required"))
+            QMessageBox.information(
+                self, self.tr("Export to file"), self.tr("Output file name is required")
+            )
             return
 
         if self.chkSourceSrid.isEnabled() and self.chkSourceSrid.isChecked():
             try:
                 sourceSrid = int(self.editSourceSrid.text())
             except ValueError:
-                QMessageBox.information(self, self.tr("Export to file"),
-                                        self.tr("Invalid source srid: must be an integer"))
+                QMessageBox.information(
+                    self,
+                    self.tr("Export to file"),
+                    self.tr("Invalid source srid: must be an integer"),
+                )
                 return
 
         if self.chkTargetSrid.isEnabled() and self.chkTargetSrid.isChecked():
             try:
                 targetSrid = int(self.editTargetSrid.text())
             except ValueError:
-                QMessageBox.information(self, self.tr("Export to file"),
-                                        self.tr("Invalid target srid: must be an integer"))
+                QMessageBox.information(
+                    self,
+                    self.tr("Export to file"),
+                    self.tr("Invalid target srid: must be an integer"),
+                )
                 return
 
-        # override cursor
-        QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
-        # store current input layer crs, so I can restore it later
-        prevInCrs = self.inLayer.crs()
-        try:
-            uri = self.editOutputFile.text()
-            providerName = "ogr"
+        with OverrideCursor(Qt.CursorShape.WaitCursor):
+            # store current input layer crs, so I can restore it later
+            prevInCrs = self.inLayer.crs()
+            try:
+                uri = self.editOutputFile.text()
+                providerName = "ogr"
 
-            options = {}
+                options = {}
 
-            # set the OGR driver will be used
-            driverName = self.cboFileFormat.itemData(self.cboFileFormat.currentIndex())
-            options['driverName'] = driverName
+                # set the OGR driver will be used
+                driverName = self.cboFileFormat.currentData()
+                options["driverName"] = driverName
 
-            # set the output file encoding
-            if self.chkEncoding.isEnabled() and self.chkEncoding.isChecked():
-                enc = self.cboEncoding.currentText()
-                options['fileEncoding'] = enc
+                # set the output file encoding
+                if self.chkEncoding.isEnabled() and self.chkEncoding.isChecked():
+                    enc = self.cboEncoding.currentText()
+                    options["fileEncoding"] = enc
 
-            if self.chkDropTable.isChecked():
-                options['overwrite'] = True
+                if self.chkDropTable.isChecked():
+                    options["overwrite"] = True
 
-            outCrs = None
-            if self.chkTargetSrid.isEnabled() and self.chkTargetSrid.isChecked():
-                targetSrid = int(self.editTargetSrid.text())
-                outCrs = QgsCoordinateReferenceSystem(targetSrid)
+                outCrs = QgsCoordinateReferenceSystem()
+                if self.chkTargetSrid.isEnabled() and self.chkTargetSrid.isChecked():
+                    targetSrid = int(self.editTargetSrid.text())
+                    outCrs = QgsCoordinateReferenceSystem(targetSrid)
 
-            # update input layer crs
-            if self.chkSourceSrid.isEnabled() and self.chkSourceSrid.isChecked():
-                sourceSrid = int(self.editSourceSrid.text())
-                inCrs = QgsCoordinateReferenceSystem(sourceSrid)
-                self.inLayer.setCrs(inCrs)
+                # update input layer crs
+                if self.chkSourceSrid.isEnabled() and self.chkSourceSrid.isChecked():
+                    sourceSrid = int(self.editSourceSrid.text())
+                    inCrs = QgsCoordinateReferenceSystem(sourceSrid)
+                    self.inLayer.setCrs(inCrs)
 
-            # do the export!
-            ret, errMsg = QgsVectorLayerImport.importLayer(self.inLayer, uri, providerName, outCrs, False,
-                                                           False, options)
-        except Exception as e:
-            ret = -1
-            errMsg = unicode(e)
+                # do the export!
+                ret, errMsg = QgsVectorLayerExporter.exportLayer(
+                    self.inLayer, uri, providerName, outCrs, False, options
+                )
+            except Exception as e:
+                ret = -1
+                errMsg = str(e)
 
-        finally:
-            # restore input layer crs and encoding
-            self.inLayer.setCrs(prevInCrs)
-            # restore cursor
-            QApplication.restoreOverrideCursor()
+            finally:
+                # restore input layer crs and encoding
+                self.inLayer.setCrs(prevInCrs)
 
         if ret != 0:
-            QMessageBox.warning(self, self.tr("Export to file"), self.tr("Error %d\n%s") % (ret, errMsg))
+            QMessageBox.warning(
+                self,
+                self.tr("Export to file"),
+                self.tr("Error {0}\n{1}").format(ret, errMsg),
+            )
             return
 
         # create spatial index
         # if self.chkSpatialIndex.isEnabled() and self.chkSpatialIndex.isChecked():
         #       self.db.connector.createSpatialIndex( (schema, table), geom )
 
-        QMessageBox.information(self, self.tr("Export to file"), self.tr("Export finished."))
+        QMessageBox.information(
+            self, self.tr("Export to file"), self.tr("Export finished.")
+        )
         return QDialog.accept(self)
-
-
-if __name__ == '__main__':
-    import sys
-
-    a = QApplication(sys.argv)
-    dlg = DlgExportVector()
-    dlg.show()
-    sys.exit(a.exec_())
