@@ -18,21 +18,21 @@
 #include "qgstininterpolator.h"
 #include "qgsfeatureiterator.h"
 #include "CloughTocherInterpolator.h"
-#include "DualEdgeTriangulation.h"
+#include "qgsdualedgetriangulation.h"
 #include "NormVecDecorator.h"
 #include "LinTriangleInterpolator.h"
 #include "qgspoint.h"
 #include "qgsfeature.h"
 #include "qgsgeometry.h"
 #include "qgsvectorlayer.h"
-#include "qgswkbptr.h"
+#include "qgsvariantutils.h"
 #include "qgsfeedback.h"
 #include "qgscurve.h"
 #include "qgsmulticurve.h"
 #include "qgscurvepolygon.h"
 #include "qgsmultisurface.h"
 
-QgsTinInterpolator::QgsTinInterpolator( const QList<LayerData> &inputData, TinInterpolation interpolation, QgsFeedback *feedback )
+QgsTinInterpolator::QgsTinInterpolator( const QList<LayerData> &inputData, QgsTinInterpolator::TinInterpolation interpolation, QgsFeedback *feedback )
   : QgsInterpolator( inputData )
   , mIsInitialized( false )
   , mFeedback( feedback )
@@ -69,7 +69,7 @@ int QgsTinInterpolator::interpolatePoint( double x, double y, double &result, Qg
 
 QgsFields QgsTinInterpolator::triangulationFields()
 {
-  return Triangulation::triangulationFields();
+  return QgsTriangulation::triangulationFields();
 }
 
 void QgsTinInterpolator::setTriangulationSink( QgsFeatureSink *sink )
@@ -79,8 +79,8 @@ void QgsTinInterpolator::setTriangulationSink( QgsFeatureSink *sink )
 
 void QgsTinInterpolator::initialize()
 {
-  DualEdgeTriangulation *dualEdgeTriangulation = new DualEdgeTriangulation( 100000, nullptr );
-  if ( mInterpolation == CloughTocher )
+  QgsDualEdgeTriangulation *dualEdgeTriangulation = new QgsDualEdgeTriangulation( 100000 );
+  if ( mInterpolation == QgsTinInterpolator::TinInterpolation::CloughTocher )
   {
     NormVecDecorator *dec = new NormVecDecorator();
     dec->addTriangulation( dualEdgeTriangulation );
@@ -92,11 +92,11 @@ void QgsTinInterpolator::initialize()
   }
 
   //get number of features if we use a progress bar
-  int nFeatures = 0;
-  int nProcessedFeatures = 0;
+  long long nFeatures = 0;
+  long long nProcessedFeatures = 0;
   if ( mFeedback )
   {
-    for ( const LayerData &layer :  qgis::as_const( mLayerData ) )
+    for ( const LayerData &layer : std::as_const( mLayerData ) )
     {
       if ( layer.source )
       {
@@ -105,24 +105,26 @@ void QgsTinInterpolator::initialize()
     }
   }
 
+  const QgsCoordinateReferenceSystem crs = !mLayerData.empty() ? mLayerData.at( 0 ).source->sourceCrs() : QgsCoordinateReferenceSystem();
+
   QgsFeature f;
-  for ( const LayerData &layer : qgis::as_const( mLayerData ) )
+  for ( const LayerData &layer : std::as_const( mLayerData ) )
   {
     if ( layer.source )
     {
       QgsAttributeList attList;
       switch ( layer.valueSource )
       {
-        case QgsInterpolator::ValueAttribute:
+        case QgsInterpolator::ValueSource::Attribute:
           attList.push_back( layer.interpolationAttribute );
           break;
 
-        case QgsInterpolator::ValueM:
-        case QgsInterpolator::ValueZ:
+        case QgsInterpolator::ValueSource::M:
+        case QgsInterpolator::ValueSource::Z:
           break;
       }
 
-      QgsFeatureIterator fit = layer.source->getFeatures( QgsFeatureRequest().setSubsetOfAttributes( attList ) );
+      QgsFeatureIterator fit = layer.source->getFeatures( QgsFeatureRequest().setSubsetOfAttributes( attList ).setDestinationCrs( crs, layer.transformContext ) );
 
       while ( fit.nextFeature( f ) )
       {
@@ -133,7 +135,7 @@ void QgsTinInterpolator::initialize()
             break;
           }
           if ( nFeatures > 0 )
-            mFeedback->setProgress( 100.0 * static_cast< double >( nProcessedFeatures ) / nFeatures );
+            mFeedback->setProgress( 100.0 * static_cast<double>( nProcessedFeatures ) / nFeatures );
         }
         insertData( f, layer.valueSource, layer.interpolationAttribute, layer.sourceType );
         ++nProcessedFeatures;
@@ -141,16 +143,16 @@ void QgsTinInterpolator::initialize()
     }
   }
 
-  if ( mInterpolation == CloughTocher )
+  if ( mInterpolation == QgsTinInterpolator::TinInterpolation::CloughTocher )
   {
-    CloughTocherInterpolator *ctInterpolator = new CloughTocherInterpolator();
     NormVecDecorator *dec = dynamic_cast<NormVecDecorator *>( mTriangulation );
     if ( dec )
     {
+      auto ctInterpolator = std::make_unique<CloughTocherInterpolator>();
       dec->estimateFirstDerivatives( mFeedback );
       ctInterpolator->setTriangulation( dec );
-      dec->setTriangleInterpolator( ctInterpolator );
-      mTriangleInterpolator = ctInterpolator;
+      mTriangleInterpolator = ctInterpolator.release();
+      dec->setTriangleInterpolator( mTriangleInterpolator );
     }
   }
   else //linear
@@ -166,7 +168,7 @@ void QgsTinInterpolator::initialize()
   }
 }
 
-int QgsTinInterpolator::insertData( const QgsFeature &f, QgsInterpolator::ValueSource source, int attr, SourceType type )
+int QgsTinInterpolator::insertData( const QgsFeature &f, QgsInterpolator::ValueSource source, int attr, QgsInterpolator::SourceType type )
 {
   QgsGeometry g = f.geometry();
   if ( g.isNull() || g.isEmpty() )
@@ -179,10 +181,10 @@ int QgsTinInterpolator::insertData( const QgsFeature &f, QgsInterpolator::ValueS
   bool attributeConversionOk = false;
   switch ( source )
   {
-    case ValueAttribute:
+    case QgsInterpolator::ValueSource::Attribute:
     {
       QVariant attributeVariant = f.attribute( attr );
-      if ( !attributeVariant.isValid() ) //attribute not found, something must be wrong (e.g. NULL value)
+      if ( QgsVariantUtils::isNull( attributeVariant ) ) //attribute not found, something must be wrong (e.g. NULL value)
       {
         return 3;
       }
@@ -194,60 +196,59 @@ int QgsTinInterpolator::insertData( const QgsFeature &f, QgsInterpolator::ValueS
       break;
     }
 
-    case ValueM:
+    case QgsInterpolator::ValueSource::M:
       if ( !g.constGet()->isMeasure() )
         return 3;
       else
         break;
 
-    case ValueZ:
+    case QgsInterpolator::ValueSource::Z:
       if ( !g.constGet()->is3D() )
         return 3;
       else
         break;
   }
 
-
   switch ( type )
   {
-    case SourcePoints:
+    case QgsInterpolator::SourceType::Points:
     {
       if ( addPointsFromGeometry( g, source, attributeValue ) != 0 )
         return -1;
       break;
     }
 
-    case SourceBreakLines:
-    case SourceStructureLines:
+    case QgsInterpolator::SourceType::BreakLines:
+    case QgsInterpolator::SourceType::StructureLines:
     {
       switch ( QgsWkbTypes::geometryType( g.wkbType() ) )
       {
-        case QgsWkbTypes::PointGeometry:
+        case Qgis::GeometryType::Point:
         {
           if ( addPointsFromGeometry( g, source, attributeValue ) != 0 )
             return -1;
           break;
         }
 
-        case QgsWkbTypes::LineGeometry:
-        case QgsWkbTypes::PolygonGeometry:
+        case Qgis::GeometryType::Line:
+        case Qgis::GeometryType::Polygon:
         {
           // need to extract all rings from input geometry
           std::vector<const QgsCurve *> curves;
-          if ( QgsWkbTypes::geometryType( g.wkbType() ) == QgsWkbTypes::PolygonGeometry )
+          if ( QgsWkbTypes::geometryType( g.wkbType() ) == Qgis::GeometryType::Polygon )
           {
-            std::vector< const QgsCurvePolygon * > polygons;
+            std::vector<const QgsCurvePolygon *> polygons;
             if ( g.isMultipart() )
             {
-              const QgsMultiSurface *ms = qgsgeometry_cast< const QgsMultiSurface * >( g.constGet() );
+              const QgsMultiSurface *ms = qgsgeometry_cast<const QgsMultiSurface *>( g.constGet() );
               for ( int i = 0; i < ms->numGeometries(); ++i )
               {
-                polygons.emplace_back( qgsgeometry_cast< const QgsCurvePolygon * >( ms->geometryN( i ) ) );
+                polygons.emplace_back( qgsgeometry_cast<const QgsCurvePolygon *>( ms->geometryN( i ) ) );
               }
             }
             else
             {
-              polygons.emplace_back( qgsgeometry_cast< const QgsCurvePolygon * >( g.constGet() ) );
+              polygons.emplace_back( qgsgeometry_cast<const QgsCurvePolygon *>( g.constGet() ) );
             }
 
             for ( const QgsCurvePolygon *polygon : polygons )
@@ -268,15 +269,15 @@ int QgsTinInterpolator::insertData( const QgsFeature &f, QgsInterpolator::ValueS
           {
             if ( g.isMultipart() )
             {
-              const QgsMultiCurve *mc = qgsgeometry_cast< const QgsMultiCurve * >( g.constGet() );
+              const QgsMultiCurve *mc = qgsgeometry_cast<const QgsMultiCurve *>( g.constGet() );
               for ( int i = 0; i < mc->numGeometries(); ++i )
               {
-                curves.emplace_back( qgsgeometry_cast< const QgsCurve * >( mc->geometryN( i ) ) );
+                curves.emplace_back( mc->curveN( i ) );
               }
             }
             else
             {
-              curves.emplace_back( qgsgeometry_cast< const QgsCurve * >( g.constGet() ) );
+              curves.emplace_back( qgsgeometry_cast<const QgsCurve *>( g.constGet() ) );
             }
           }
 
@@ -285,34 +286,36 @@ int QgsTinInterpolator::insertData( const QgsFeature &f, QgsInterpolator::ValueS
             if ( !curve )
               continue;
 
-            QVector< QgsPoint > linePoints;
-            for ( auto point = g.vertices_begin(); point != g.vertices_end(); ++point )
+            QgsPointSequence linePoints;
+            curve->points( linePoints );
+            for ( QgsPoint &point : linePoints )
             {
-              QgsPoint p = *point;
-              double z = 0;
               switch ( source )
               {
-                case ValueAttribute:
-                  z = attributeValue;
+                case QgsInterpolator::ValueSource::Attribute:
+                  if ( point.is3D() )
+                    point.setZ( attributeValue );
+                  else
+                    point.addZValue( attributeValue );
                   break;
 
-                case ValueZ:
-                  z = p.z();
+                case QgsInterpolator::ValueSource::M:
+                  if ( point.is3D() )
+                    point.setZ( point.m() );
+                  else
+                    point.addZValue( point.m() );
                   break;
 
-                case ValueM:
-                  z = p.m();
+                case QgsInterpolator::ValueSource::Z:
                   break;
               }
-
-              linePoints.append( QgsPoint( p.x(), p.y(), z ) );
             }
             mTriangulation->addLine( linePoints, type );
           }
           break;
         }
-        case QgsWkbTypes::UnknownGeometry:
-        case QgsWkbTypes::NullGeometry:
+        case Qgis::GeometryType::Unknown:
+        case Qgis::GeometryType::Null:
           break;
       }
       break;
@@ -323,7 +326,7 @@ int QgsTinInterpolator::insertData( const QgsFeature &f, QgsInterpolator::ValueS
 }
 
 
-int QgsTinInterpolator::addPointsFromGeometry( const QgsGeometry &g, ValueSource source, double attributeValue )
+int QgsTinInterpolator::addPointsFromGeometry( const QgsGeometry &g, QgsInterpolator::ValueSource source, double attributeValue )
 {
   // loop through all vertices and add to triangulation
   for ( auto point = g.vertices_begin(); point != g.vertices_end(); ++point )
@@ -332,15 +335,15 @@ int QgsTinInterpolator::addPointsFromGeometry( const QgsGeometry &g, ValueSource
     double z = 0;
     switch ( source )
     {
-      case ValueAttribute:
+      case QgsInterpolator::ValueSource::Attribute:
         z = attributeValue;
         break;
 
-      case ValueZ:
+      case QgsInterpolator::ValueSource::Z:
         z = p.z();
         break;
 
-      case ValueM:
+      case QgsInterpolator::ValueSource::M:
         z = p.m();
         break;
     }
