@@ -16,40 +16,54 @@
  ***************************************************************************/
 
 #include "qgsabout.h"
+#include "moc_qgsabout.cpp"
 #include "qgsapplication.h"
 #include "qgsauthmethodregistry.h"
 #include "qgsproviderregistry.h"
 #include "qgslogger.h"
+#include <QClipboard>
 #include <QDesktopServices>
 #include <QFile>
 #include <QTextStream>
 #include <QImageReader>
 #include <QSqlDatabase>
 #include <QTcpSocket>
+#include <QUrl>
+#include <QRegularExpression>
 
-#ifdef Q_OS_MACX
-QgsAbout::QgsAbout( QWidget *parent )
-    : QgsOptionsDialogBase( "about", parent, Qt::WindowSystemMenuHint )  // Modeless dialog with close button only
+#ifdef Q_OS_MACOS
+// Modeless dialog with close button only
+constexpr Qt::WindowFlags kAboutWindowFlags = Qt::WindowSystemMenuHint;
 #else
-QgsAbout::QgsAbout( QWidget *parent )
-    : QgsOptionsDialogBase( "about", parent )  // Normal dialog in non Mac-OS
+// Normal dialog in non Mac-OS
+constexpr Qt::WindowFlags kAboutWindowFlags = Qt::WindowFlags();
 #endif
+
+QgsAbout::QgsAbout( QWidget *parent )
+  : QgsOptionsDialogBase( QStringLiteral( "about" ), parent, kAboutWindowFlags )
 {
   setupUi( this );
-  initOptionsBase( true, QString( "%1 - %2 Bit" ).arg( windowTitle() ).arg( QSysInfo::WordSize ) );
+  connect( btnQgisUser, &QPushButton::clicked, this, &QgsAbout::btnQgisUser_clicked );
+  connect( btnQgisHome, &QPushButton::clicked, this, &QgsAbout::btnQgisHome_clicked );
+  connect( btnCopyToClipboard, &QPushButton::clicked, this, &QgsAbout::btnCopyToClipboard_clicked );
+  if constexpr ( QSysInfo::WordSize != 64 )
+  {
+    // 64 bit is the current standard. Only specify word size if it is not 64.
+    initOptionsBase( true, tr( "%1 - %2 Bit" ).arg( windowTitle() ).arg( QSysInfo::WordSize ) );
+  }
+  else
+  {
+    initOptionsBase( true );
+  }
   init();
 }
 
-QgsAbout::~QgsAbout()
-{
-}
-
 void QgsAbout::init()
 {
   setPluginInfo();
 
   // check internet connection in order to hide/show the developers map widget
-  int DEVELOPERS_MAP_INDEX = 5;
+  const int DEVELOPERS_MAP_INDEX = 5;
   QTcpSocket socket;
   socket.connectToHost( QgsApplication::QGIS_ORGANIZATION_DOMAIN, 80 );
   if ( socket.waitForConnected( 1000 ) )
@@ -59,16 +73,13 @@ void QgsAbout::init()
   else
   {
     mOptionsListWidget->item( DEVELOPERS_MAP_INDEX )->setHidden( true );
-    QModelIndex firstItem = mOptionsListWidget->model()->index( 0, 0, QModelIndex() );
+    const QModelIndex firstItem = mOptionsListWidget->model()->index( 0, 0, QModelIndex() );
     mOptionsListWidget->setCurrentIndex( firstItem );
   }
   developersMapView->page()->setLinkDelegationPolicy( QWebPage::DelegateAllLinks );
   developersMapView->setContextMenuPolicy( Qt::NoContextMenu );
 
-  connect( developersMapView, SIGNAL( linkClicked( const QUrl & ) ), this, SLOT( openUrl( const QUrl & ) ) );
-
-  // set the 60x60 icon pixmap
-  qgisIcon->setPixmap( QPixmap( QgsApplication::appIconPath() ) );
+  connect( developersMapView, &QgsWebView::linkClicked, this, &QgsAbout::openUrl );
 
   //read the authors file to populate the svn committers list
   QStringList lines;
@@ -80,16 +91,18 @@ void QgsAbout::init()
   if ( file.open( QIODevice::ReadOnly ) )
   {
     QTextStream stream( &file );
+#if QT_VERSION < QT_VERSION_CHECK( 6, 0, 0 )
     // Always use UTF-8
     stream.setCodec( "UTF-8" );
+#endif
     QString line;
     while ( !stream.atEnd() )
     {
       line = stream.readLine(); // line of text excluding '\n'
       //ignore the line if it starts with a hash....
-      if ( line.left( 1 ) == "#" )
+      if ( !line.isEmpty() && line.at( 0 ) == '#' )
         continue;
-      QStringList myTokens = line.split( '\t', QString::SkipEmptyParts );
+      QStringList myTokens = line.split( '\t', Qt::SkipEmptyParts );
       lines << myTokens[0];
     }
     file.close();
@@ -107,19 +120,19 @@ void QgsAbout::init()
   // Now load up the contributors list
   //
   QFile file2( QgsApplication::contributorsFilePath() );
-  printf( "Reading contributors file %s.............................................\n",
-          file2.fileName().toLocal8Bit().constData() );
   if ( file2.open( QIODevice::ReadOnly ) )
   {
     QTextStream stream( &file2 );
+#if QT_VERSION < QT_VERSION_CHECK( 6, 0, 0 )
     // Always use UTF-8
     stream.setCodec( "UTF-8" );
+#endif
     QString line;
     while ( !stream.atEnd() )
     {
       line = stream.readLine(); // line of text excluding '\n'
       //ignore the line if it starts with a hash....
-      if ( line.left( 1 ) == "#" )
+      if ( !line.isEmpty() && line.at( 0 ) == '#' )
         continue;
       lines += line;
     }
@@ -132,21 +145,14 @@ void QgsAbout::init()
     }
   }
 
-
-
   // read the DONORS file and populate the text widget
   QFile donorsFile( QgsApplication::donorsFilePath() );
-#ifdef QGISDEBUG
-  printf( "Reading donors file %s.............................................\n",
-          donorsFile.fileName().toLocal8Bit().constData() );
-#endif
   if ( donorsFile.open( QIODevice::ReadOnly ) )
   {
-    QString donorsHTML = ""
-                         + tr( "<p>For a list of individuals and institutions who have contributed "
-                               "money to fund QGIS development and other project costs see "
-                               "<a href=\"http://qgis.org/en/site/about/sponsorship.html#list-of-donors\">"
-                               "http://qgis.org/en/site/about/sponsorship.html#list-of-donors</a></p>" );
+    const QString donorsHTML = tr( "<p>For a list of individuals and institutions who have contributed "
+                                   "money to fund QGIS development and other project costs see "
+                                   "<a href=\"https://qgis.org/funding/donate/\">"
+                                   "https://qgis.org/funding/donate/</a></p>" );
 #if 0
     QString website;
     QTextStream donorsStream( &donorsFile );
@@ -179,29 +185,27 @@ void QgsAbout::init()
     txtDonors->clear();
     txtDonors->document()->setDefaultStyleSheet( QgsApplication::reportStyleSheet() );
     txtDonors->setHtml( donorsHTML );
-    QgsDebugMsg( QString( "donorsHTML:%1" ).arg( donorsHTML.toAscii().constData() ) );
+    QgsDebugMsgLevel( QStringLiteral( "donorsHTML:%1" ).arg( donorsHTML.toLatin1().constData() ), 2 );
   }
 
   // read the TRANSLATORS file and populate the text widget
   QFile translatorFile( QgsApplication::translatorsFilePath() );
-#ifdef QGISDEBUG
-  printf( "Reading translators file %s.............................................\n",
-          translatorFile.fileName().toLocal8Bit().constData() );
-#endif
   if ( translatorFile.open( QIODevice::ReadOnly ) )
   {
-    QString translatorHTML = "";
+    QString translatorHTML;
     QTextStream translatorStream( &translatorFile );
     // Always use UTF-8
+#if QT_VERSION < QT_VERSION_CHECK( 6, 0, 0 )
     translatorStream.setCodec( "UTF-8" );
-    QString myStyle = QgsApplication::reportStyleSheet();
+#endif
+    const QString myStyle = QgsApplication::reportStyleSheet();
     translatorHTML += "<style>" + myStyle + "</style>";
     while ( !translatorStream.atEnd() )
     {
       translatorHTML += translatorStream.readLine();
     }
     txtTranslators->setHtml( translatorHTML );
-    QgsDebugMsg( QString( "translatorHTML:%1" ).arg( translatorHTML.toAscii().constData() ) );
+    QgsDebugMsgLevel( QStringLiteral( "translatorHTML:%1" ).arg( translatorHTML.toLatin1().constData() ), 2 );
   }
   setWhatsNew();
   setLicence();
@@ -211,28 +215,29 @@ void QgsAbout::setLicence()
 {
   // read the DONORS file and populate the text widget
   QFile licenceFile( QgsApplication::licenceFilePath() );
-#ifdef QGISDEBUG
-  printf( "Reading licence file %s.............................................\n",
-          licenceFile.fileName().toLocal8Bit().constData() );
-#endif
+  QgsDebugMsgLevel( QStringLiteral( "Reading licence file %1" ).arg( licenceFile.fileName() ), 2 );
   if ( licenceFile.open( QIODevice::ReadOnly ) )
   {
     txtLicense->setText( licenceFile.readAll() );
   }
 }
 
-void QgsAbout::setVersion( const QString& v )
+void QgsAbout::setVersion( const QString &v )
 {
   txtVersion->setBackgroundRole( QPalette::NoRole );
   txtVersion->setAutoFillBackground( true );
   txtVersion->setHtml( v );
+  mVersionString = v;
 }
 
 void QgsAbout::setWhatsNew()
 {
   txtWhatsNew->clear();
   txtWhatsNew->document()->setDefaultStyleSheet( QgsApplication::reportStyleSheet() );
-  txtWhatsNew->setSource( "file:///" + QgsApplication::pkgDataPath() + "/doc/news.html" );
+  if ( !QFile::exists( QgsApplication::pkgDataPath() + "/doc/NEWS.html" ) )
+    return;
+
+  txtWhatsNew->setSource( QString( "file:///" + QgsApplication::pkgDataPath() + "/doc/NEWS.html" ) );
 }
 
 void QgsAbout::setPluginInfo()
@@ -245,39 +250,44 @@ void QgsAbout::setPluginInfo()
   myString += QgsAuthMethodRegistry::instance()->pluginList( true );
   //qt database plugins
   myString += "<b>" + tr( "Available Qt Database Plugins" ) + "</b><br>";
-  myString += "<ol>\n<li>\n";
-  QStringList myDbDriverList = QSqlDatabase::drivers();
-  myString += myDbDriverList.join( "</li>\n<li>" );
-  myString += "</li>\n</ol>\n";
+  myString += QLatin1String( "<ol>\n<li>\n" );
+  const QStringList myDbDriverList = QSqlDatabase::drivers();
+  myString += myDbDriverList.join( QLatin1String( "</li>\n<li>" ) );
+  myString += QLatin1String( "</li>\n</ol>\n" );
   //qt image plugins
   myString += "<b>" + tr( "Available Qt Image Plugins" ) + "</b><br>";
   myString += tr( "Qt Image Plugin Search Paths <br>" );
-  myString += QApplication::libraryPaths().join( "<br>" );
-  myString += "<ol>\n<li>\n";
-  QList<QByteArray> myImageFormats = QImageReader::supportedImageFormats();
-  QList<QByteArray>::const_iterator myIterator = myImageFormats.begin();
-  while ( myIterator != myImageFormats.end() )
+  myString += QApplication::libraryPaths().join( QLatin1String( "<br>" ) );
+  myString += QLatin1String( "<ol>\n<li>\n" );
+  const QList<QByteArray> myImageFormats = QImageReader::supportedImageFormats();
+  QList<QByteArray>::const_iterator myIterator = myImageFormats.constBegin();
+  while ( myIterator != myImageFormats.constEnd() )
   {
-    QString myFormat = ( *myIterator ).data();
+    const QString myFormat = ( *myIterator ).data();
     myString += myFormat + "</li>\n<li>";
     ++myIterator;
   }
-  myString += "</li>\n</ol>\n";
+  myString += QLatin1String( "</li>\n</ol>\n" );
 
-  QString myStyle = QgsApplication::reportStyleSheet();
+  const QString myStyle = QgsApplication::reportStyleSheet();
   txtProviders->clear();
   txtProviders->document()->setDefaultStyleSheet( myStyle );
   txtProviders->setText( myString );
 }
 
-void QgsAbout::on_btnQgisUser_clicked()
+void QgsAbout::btnCopyToClipboard_clicked()
 {
-  openUrl( QString( "http://lists.osgeo.org/mailman/listinfo/qgis-user" ) );
+  QGuiApplication::clipboard()->setText( mVersionString );
 }
 
-void QgsAbout::on_btnQgisHome_clicked()
+void QgsAbout::btnQgisUser_clicked()
 {
-  openUrl( QString( "http://qgis.org" ) );
+  openUrl( QStringLiteral( "https://lists.osgeo.org/mailman/listinfo/qgis-user" ) );
+}
+
+void QgsAbout::btnQgisHome_clicked()
+{
+  openUrl( QStringLiteral( "https://qgis.org" ) );
 }
 
 void QgsAbout::openUrl( const QUrl &url )
@@ -292,26 +302,28 @@ void QgsAbout::openUrl( const QUrl &url )
  * Step 2: Replace all bytes of the UTF-8 above 0x7f with the hexcode in lower case.
  * Step 2: Replace all non [a-z][a-Z][0-9] with underscore (backward compatibility)
  */
-QString QgsAbout::fileSystemSafe( const QString& fileName )
+QString QgsAbout::fileSystemSafe( const QString &fileName )
 {
   QString result;
   QByteArray utf8 = fileName.toUtf8();
 
   for ( int i = 0; i < utf8.size(); i++ )
   {
-    uchar c = utf8[i];
+    const uchar c = utf8[i];
 
     if ( c > 0x7f )
     {
-      result = result + QString( "%1" ).arg( c, 2, 16, QChar( '0' ) );
+      result = result + QStringLiteral( "%1" ).arg( c, 2, 16, QChar( '0' ) );
     }
     else
     {
-      result = result + QString( c );
+      result = result + QChar( c );
     }
   }
-  result.replace( QRegExp( "[^a-z0-9A-Z]" ), "_" );
-  QgsDebugMsg( result );
+
+  const thread_local QRegularExpression sNonAlphaNumericRx( QStringLiteral( "[^a-zA-Z0-9]" ) );
+  result.replace( sNonAlphaNumericRx, QStringLiteral( "_" ) );
+  QgsDebugMsgLevel( result, 3 );
 
   return result;
 }
@@ -319,6 +331,6 @@ QString QgsAbout::fileSystemSafe( const QString& fileName )
 void QgsAbout::setDevelopersMap()
 {
   developersMapView->settings()->setAttribute( QWebSettings::JavascriptEnabled, true );
-  QUrl url = QUrl::fromLocalFile( QgsApplication::developersMapFilePath() );
+  const QUrl url = QUrl::fromLocalFile( QgsApplication::developersMapFilePath() );
   developersMapView->load( url );
 }
