@@ -40,27 +40,6 @@ QgsCompoundCurve::~QgsCompoundCurve()
   clear();
 }
 
-bool QgsCompoundCurve::equals( const QgsCurve &other ) const
-{
-  const QgsCompoundCurve *otherCurve = qgsgeometry_cast< const QgsCompoundCurve * >( &other );
-  if ( !otherCurve )
-    return false;
-
-  if ( mWkbType != otherCurve->mWkbType )
-    return false;
-
-  if ( mCurves.size() != otherCurve->mCurves.size() )
-    return false;
-
-  for ( int i = 0; i < mCurves.size(); ++i )
-  {
-    if ( *mCurves.at( i ) != *otherCurve->mCurves.at( i ) )
-      return false;
-  }
-
-  return true;
-}
-
 QgsCompoundCurve *QgsCompoundCurve::createEmptyWithSameType() const
 {
   auto result = std::make_unique< QgsCompoundCurve >();
@@ -486,13 +465,13 @@ QgsLineString *QgsCompoundCurve::curveToLine( double tolerance, SegmentationTole
   return line;
 }
 
-QgsCompoundCurve *QgsCompoundCurve::snappedToGrid( double hSpacing, double vSpacing, double dSpacing, double mSpacing ) const
+QgsCompoundCurve *QgsCompoundCurve::snappedToGrid( double hSpacing, double vSpacing, double dSpacing, double mSpacing, bool removeRedundantPoints ) const
 {
   std::unique_ptr<QgsCompoundCurve> result( createEmptyWithSameType() );
 
   for ( QgsCurve *curve : mCurves )
   {
-    std::unique_ptr<QgsCurve> gridified( static_cast< QgsCurve * >( curve->snappedToGrid( hSpacing, vSpacing, dSpacing, mSpacing ) ) );
+    std::unique_ptr<QgsCurve> gridified( static_cast< QgsCurve * >( curve->snappedToGrid( hSpacing, vSpacing, dSpacing, mSpacing, removeRedundantPoints ) ) );
     if ( gridified )
     {
       result->mCurves.append( gridified.release() );
@@ -505,6 +484,12 @@ QgsCompoundCurve *QgsCompoundCurve::snappedToGrid( double hSpacing, double vSpac
     return result.release();
 }
 
+QgsAbstractGeometry *QgsCompoundCurve::simplifyByDistance( double tolerance ) const
+{
+  std::unique_ptr< QgsLineString > line( curveToLine() );
+  return line->simplifyByDistance( tolerance );
+}
+
 bool QgsCompoundCurve::removeDuplicateNodes( double epsilon, bool useZValues )
 {
   bool result = false;
@@ -817,20 +802,20 @@ bool QgsCompoundCurve::deleteVertex( QgsVertexId position )
 
         if ( subVertexId.vertex < points.length() - 2 )
         {
-          std::unique_ptr<QgsCircularString> curveC = std::make_unique<QgsCircularString>();
+          auto curveC = std::make_unique<QgsCircularString>();
           curveC->setPoints( points.mid( subVertexId.vertex + 1 ) );
           mCurves.insert( curveId, curveC.release() );
         }
 
         const QgsPointSequence partB = QgsPointSequence() << points[subVertexId.vertex - 1] << points[subVertexId.vertex + 1];
-        std::unique_ptr<QgsLineString> curveB = std::make_unique<QgsLineString>();
+        auto curveB = std::make_unique<QgsLineString>();
         curveB->setPoints( partB );
         mCurves.insert( curveId, curveB.release() );
         curve = mCurves.at( curveId );
 
         if ( subVertexId.vertex > 1 )
         {
-          std::unique_ptr<QgsCircularString> curveA = std::make_unique<QgsCircularString>();
+          auto curveA = std::make_unique<QgsCircularString>();
           curveA->setPoints( points.mid( 0, subVertexId.vertex ) );
           mCurves.insert( curveId, curveA.release() );
         }
@@ -1019,11 +1004,11 @@ bool QgsCompoundCurve::toggleCircularAtVertex( QgsVertexId position )
     const QgsPointSequence partB  = QgsPointSequence() << points[subVertexId.vertex - 1] << points[subVertexId.vertex] << points[subVertexId.vertex + 1];
     const QgsPointSequence partC  = points.mid( subVertexId.vertex + 1 );
 
-    std::unique_ptr<QgsCircularString> curveA = std::make_unique<QgsCircularString>();
+    auto curveA = std::make_unique<QgsCircularString>();
     curveA->setPoints( partA );
-    std::unique_ptr<QgsLineString> curveB = std::make_unique<QgsLineString>();
+    auto curveB = std::make_unique<QgsLineString>();
     curveB->setPoints( partB );
-    std::unique_ptr<QgsCircularString> curveC = std::make_unique<QgsCircularString>();
+    auto curveC = std::make_unique<QgsCircularString>();
     curveC->setPoints( partC );
 
     removeCurve( curveId );
@@ -1044,11 +1029,11 @@ bool QgsCompoundCurve::toggleCircularAtVertex( QgsVertexId position )
     const QgsPointSequence partB  = QgsPointSequence() << points[subVertexId.vertex - 1] << points[subVertexId.vertex] << points[subVertexId.vertex + 1];
     const QgsPointSequence partC  = points.mid( subVertexId.vertex + 1 );
 
-    std::unique_ptr<QgsLineString> curveA = std::make_unique<QgsLineString>();
+    auto curveA = std::make_unique<QgsLineString>();
     curveA->setPoints( partA );
-    std::unique_ptr<QgsCircularString> curveB = std::make_unique<QgsCircularString>();
+    auto curveB = std::make_unique<QgsCircularString>();
     curveB->setPoints( partB );
-    std::unique_ptr<QgsLineString> curveC = std::make_unique<QgsLineString>();
+    auto curveC = std::make_unique<QgsLineString>();
     curveC->setPoints( partC );
 
     removeCurve( curveId );
@@ -1193,7 +1178,7 @@ std::tuple<std::unique_ptr<QgsCurve>, std::unique_ptr<QgsCurve> > QgsCompoundCur
 
   int curveStart = 0;
 
-  std::unique_ptr< QgsCompoundCurve > curve1 = std::make_unique< QgsCompoundCurve >();
+  auto curve1 = std::make_unique< QgsCompoundCurve >();
   std::unique_ptr< QgsCompoundCurve > curve2;
 
   for ( const QgsCurve *curve : mCurves )
@@ -1278,7 +1263,7 @@ double QgsCompoundCurve::vertexAngle( QgsVertexId vertex ) const
     QgsCurve *curve2 = mCurves[curveIds.at( 1 ).first];
     double angle1 = curve1->vertexAngle( curveIds.at( 0 ).second );
     double angle2 = curve2->vertexAngle( curveIds.at( 1 ).second );
-    return QgsGeometryUtils::averageAngle( angle1, angle2 );
+    return QgsGeometryUtilsBase::averageAngle( angle1, angle2 );
   }
   else
   {
@@ -1338,7 +1323,7 @@ QgsCompoundCurve *QgsCompoundCurve::curveSubstring( double startDistance, double
     return createEmptyWithSameType();
 
   endDistance = std::max( startDistance, endDistance );
-  std::unique_ptr< QgsCompoundCurve > substring = std::make_unique< QgsCompoundCurve >();
+  auto substring = std::make_unique< QgsCompoundCurve >();
 
   double distanceTraversed = 0;
   for ( const QgsCurve *curve : mCurves )
