@@ -41,12 +41,11 @@ class QgsTextDocumentMetrics;
 
 /**
  * \ingroup core
- * \brief The QgsLegendRendererItem class is abstract interface for legend items
+ * \brief An abstract interface for legend items
  * returned from QgsMapLayerLegend implementation.
  *
  * The objects are used in QgsLayerTreeModel. Custom implementations may offer additional interactivity
  * and customized look.
- *
  */
 class CORE_EXPORT QgsLayerTreeModelLegendNode : public QObject
 {
@@ -274,11 +273,9 @@ class CORE_EXPORT QgsLayerTreeModelLegendNode : public QObject
 
     /**
      * Entry point called from QgsLegendRenderer to do the rendering.
-     *  Default implementation calls drawSymbol() and drawSymbolText() methods.
-     *
-     *  If ctx is NULLPTR, this is just first stage when preparing layout - without actual rendering.
+     * Default implementation calls drawSymbol() and drawSymbolText() methods.
      */
-    virtual ItemMetrics draw( const QgsLegendSettings &settings, ItemContext *ctx );
+    virtual ItemMetrics draw( const QgsLegendSettings &settings, ItemContext &ctx );
 
     /**
      * Entry point called from QgsLegendRenderer to do the rendering in a
@@ -584,7 +581,7 @@ class CORE_EXPORT QgsSymbolLegendNode : public QgsLayerTreeModelLegendNode
 
 /**
  * \ingroup core
- * \brief Implementation of legend node interface for displaying arbitrary label with icon.
+ * \brief Implementation of legend node interface for displaying arbitrary labels with icons.
  *
  */
 class CORE_EXPORT QgsSimpleLegendNode : public QgsLayerTreeModelLegendNode
@@ -623,8 +620,7 @@ class CORE_EXPORT QgsSimpleLegendNode : public QgsLayerTreeModelLegendNode
 
 /**
  * \ingroup core
- * \brief Implementation of legend node interface for displaying arbitrary raster image
- *
+ * \brief Implementation of legend node interface for displaying arbitrary raster images.
  */
 class CORE_EXPORT QgsImageLegendNode : public QgsLayerTreeModelLegendNode
 {
@@ -660,7 +656,7 @@ class CORE_EXPORT QgsImageLegendNode : public QgsLayerTreeModelLegendNode
 
 /**
  * \ingroup core
- * \brief Implementation of legend node interface for displaying raster legend entries
+ * \brief Implementation of legend node interface for displaying raster legend entries.
  *
  */
 class CORE_EXPORT QgsRasterSymbolLegendNode : public QgsLayerTreeModelLegendNode
@@ -722,7 +718,7 @@ class QgsImageFetcher;
 
 /**
  * \ingroup core
- * \brief Implementation of legend node interface for displaying WMS legend entries
+ * \brief Implementation of legend node interface for displaying WMS legend entries.
  *
  */
 class CORE_EXPORT QgsWmsLegendNode : public QgsLayerTreeModelLegendNode
@@ -786,7 +782,7 @@ class CORE_EXPORT QgsWmsLegendNode : public QgsLayerTreeModelLegendNode
 
 /**
  * \ingroup core
- * \brief Produces legend node with a marker symbol
+ * \brief A legend node with a marker symbol.
  */
 class CORE_EXPORT QgsDataDefinedSizeLegendNode : public QgsLayerTreeModelLegendNode
 {
@@ -799,7 +795,7 @@ class CORE_EXPORT QgsDataDefinedSizeLegendNode : public QgsLayerTreeModelLegendN
 
     QVariant data( int role ) const override;
 
-    ItemMetrics draw( const QgsLegendSettings &settings, ItemContext *ctx ) override;
+    ItemMetrics draw( const QgsLegendSettings &settings, ItemContext &ctx ) override;
 
 #ifdef SIP_RUN
     SIP_PYOBJECT __repr__();
@@ -811,13 +807,13 @@ class CORE_EXPORT QgsDataDefinedSizeLegendNode : public QgsLayerTreeModelLegendN
 
   private:
     void cacheImage() const;
-    QgsDataDefinedSizeLegend *mSettings = nullptr;
+    std::unique_ptr<QgsDataDefinedSizeLegend> mSettings;
     mutable QImage mImage;
 };
 
 /**
  * \ingroup core
- * \brief Produces legend node for a labeling text symbol
+ * \brief A legend node for a labeling text symbol.
  * \since QGIS 3.20
  */
 class CORE_EXPORT QgsVectorLabelLegendNode : public QgsLayerTreeModelLegendNode
