@@ -16,6 +16,7 @@
  ***************************************************************************/
 
 #include "qgslayoutitemattributetable.h"
+#include "moc_qgslayoutitemattributetable.cpp"
 #include "qgslayout.h"
 #include "qgslayouttablecolumn.h"
 #include "qgslayoutitemmap.h"
@@ -25,48 +26,18 @@
 #include "qgslayoutframe.h"
 #include "qgsproject.h"
 #include "qgsrelationmanager.h"
+#include "qgsfieldformatter.h"
+#include "qgsfieldformatterregistry.h"
 #include "qgsgeometry.h"
 #include "qgsexception.h"
-#include "qgsmapsettings.h"
-
-//QgsLayoutAttributeTableCompare
-
-///@cond PRIVATE
-
-/**
- * Helper class for sorting tables, takes into account sorting column and ascending / descending
-*/
-class CORE_EXPORT QgsLayoutAttributeTableCompare
-{
-  public:
-
-    /**
-     * Constructor for QgsLayoutAttributeTableCompare.
-     */
-    QgsLayoutAttributeTableCompare() = default;
-    bool operator()( const QgsLayoutTableRow &m1, const QgsLayoutTableRow &m2 )
-    {
-      return ( mAscending ? qgsVariantLessThan( m1[mCurrentSortColumn], m2[mCurrentSortColumn] )
-               : qgsVariantGreaterThan( m1[mCurrentSortColumn], m2[mCurrentSortColumn] ) );
-    }
-
-    /**
-     * Sets \a column number to sort by.
-     */
-    void setSortColumn( int column ) { mCurrentSortColumn = column; }
-
-    /**
-     * Sets sort order for column sorting
-     * Set \a ascending to true to sort in ascending order, false to sort in descending order
-     */
-    void setAscending( bool ascending ) { mAscending = ascending; }
-
-  private:
-    int mCurrentSortColumn = 0;
-    bool mAscending = true;
-};
-
-///@endcond
+#include "qgslayoutreportcontext.h"
+#include "qgsexpressioncontextutils.h"
+#include "qgsexpressionnodeimpl.h"
+#include "qgsgeometryengine.h"
+#include "qgsconditionalstyle.h"
+#include "qgsfontutils.h"
+#include "qgsvariantutils.h"
+#include "qgslayoutrendercontext.h"
 
 //
 // QgsLayoutItemAttributeTable
@@ -182,10 +153,15 @@ void QgsLayoutItemAttributeTable::atlasLayerChanged( QgsVectorLayer *layer )
     disconnect( mCurrentAtlasLayer, &QgsVectorLayer::layerModified, this, &QgsLayoutTable::refreshAttributes );
   }
 
+  const bool mustRebuildColumns = static_cast< bool >( mCurrentAtlasLayer ) || mColumns.empty();
   mCurrentAtlasLayer = layer;
 
-  //rebuild column list to match all columns from layer
-  resetColumns();
+  if ( mustRebuildColumns )
+  {
+    //rebuild column list to match all columns from layer
+    resetColumns();
+  }
+
   refreshAttributes();
 
   //listen for modifications to layer and refresh table when they occur
@@ -201,23 +177,54 @@ void QgsLayoutItemAttributeTable::resetColumns()
   }
 
   //remove existing columns
-  qDeleteAll( mColumns );
   mColumns.clear();
+  mSortColumns.clear();
 
   //rebuild columns list from vector layer fields
   int idx = 0;
   const QgsFields sourceFields = source->fields();
+
   for ( const auto &field : sourceFields )
   {
     QString currentAlias = source->attributeDisplayName( idx );
-    std::unique_ptr< QgsLayoutTableColumn > col = qgis::make_unique< QgsLayoutTableColumn >();
-    col->setAttribute( field.name() );
-    col->setHeading( currentAlias );
-    mColumns.append( col.release() );
+    QgsLayoutTableColumn col;
+    col.setAttribute( field.name() );
+    col.setHeading( currentAlias );
+    mColumns.append( col );
     idx++;
   }
 }
 
+void QgsLayoutItemAttributeTable::disconnectCurrentMap()
+{
+  if ( !mMap )
+  {
+    return;
+  }
+
+  disconnect( mMap, &QgsLayoutItemMap::extentChanged, this, &QgsLayoutTable::refreshAttributes );
+  disconnect( mMap, &QgsLayoutItemMap::mapRotationChanged, this, &QgsLayoutTable::refreshAttributes );
+  disconnect( mMap, &QObject::destroyed, this, &QgsLayoutItemAttributeTable::disconnectCurrentMap );
+  mMap = nullptr;
+}
+
+bool QgsLayoutItemAttributeTable::useConditionalStyling() const
+{
+  return mUseConditionalStyling;
+}
+
+void QgsLayoutItemAttributeTable::setUseConditionalStyling( bool useConditionalStyling )
+{
+  if ( useConditionalStyling == mUseConditionalStyling )
+  {
+    return;
+  }
+
+  mUseConditionalStyling = useConditionalStyling;
+  refreshAttributes();
+  emit changed();
+}
+
 void QgsLayoutItemAttributeTable::setMap( QgsLayoutItemMap *map )
 {
   if ( map == mMap )
@@ -225,17 +232,14 @@ void QgsLayoutItemAttributeTable::setMap( QgsLayoutItemMap *map )
     //no change
     return;
   }
+  disconnectCurrentMap();
 
-  if ( mMap )
-  {
-    //disconnect from previous map
-    disconnect( mMap, &QgsLayoutItemMap::extentChanged, this, &QgsLayoutTable::refreshAttributes );
-  }
   mMap = map;
   if ( mMap )
   {
     //listen out for extent changes in linked map
     connect( mMap, &QgsLayoutItemMap::extentChanged, this, &QgsLayoutTable::refreshAttributes );
+    connect( mMap, &QgsLayoutItemMap::mapRotationChanged, this, &QgsLayoutTable::refreshAttributes );
   }
   refreshAttributes();
   emit changed();
@@ -322,7 +326,6 @@ void QgsLayoutItemAttributeTable::setDisplayedFields( const QStringList &fields,
   }
 
   //rebuild columns list, taking only fields contained in supplied list
-  qDeleteAll( mColumns );
   mColumns.clear();
 
   const QgsFields layerFields = source->fields();
@@ -333,13 +336,14 @@ void QgsLayoutItemAttributeTable::setDisplayedFields( const QStringList &fields,
     {
       int attrIdx = layerFields.lookupField( field );
       if ( attrIdx < 0 )
+      {
         continue;
-
+      }
       QString currentAlias = source->attributeDisplayName( attrIdx );
-      std::unique_ptr< QgsLayoutTableColumn > col = qgis::make_unique< QgsLayoutTableColumn >();
-      col->setAttribute( layerFields.at( attrIdx ).name() );
-      col->setHeading( currentAlias );
-      mColumns.append( col.release() );
+      QgsLayoutTableColumn col;
+      col.setAttribute( layerFields.at( attrIdx ).name() );
+      col.setHeading( currentAlias );
+      mColumns.append( col );
     }
   }
   else
@@ -349,10 +353,10 @@ void QgsLayoutItemAttributeTable::setDisplayedFields( const QStringList &fields,
     for ( const QgsField &field : layerFields )
     {
       QString currentAlias = source->attributeDisplayName( idx );
-      std::unique_ptr< QgsLayoutTableColumn > col = qgis::make_unique< QgsLayoutTableColumn >();
-      col->setAttribute( field.name() );
-      col->setHeading( currentAlias );
-      mColumns.append( col.release() );
+      QgsLayoutTableColumn col;
+      col.setAttribute( field.name() );
+      col.setHeading( currentAlias );
+      mColumns.append( col );
       idx++;
     }
   }
@@ -371,16 +375,16 @@ void QgsLayoutItemAttributeTable::restoreFieldAliasMap( const QMap<int, QString>
     return;
   }
 
-  for ( QgsLayoutTableColumn *column : qgis::as_const( mColumns ) )
+  for ( int i = 0; i < mColumns.count(); i++ )
   {
-    int attrIdx = source->fields().lookupField( column->attribute() );
+    int attrIdx = source->fields().lookupField( mColumns[i].attribute() );
     if ( map.contains( attrIdx ) )
     {
-      column->setHeading( map.value( attrIdx ) );
+      mColumns[i].setHeading( map.value( attrIdx ) );
     }
     else
     {
-      column->setHeading( source->attributeDisplayName( attrIdx ) );
+      mColumns[i].setHeading( source->attributeDisplayName( attrIdx ) );
     }
   }
 }
@@ -388,6 +392,7 @@ void QgsLayoutItemAttributeTable::restoreFieldAliasMap( const QMap<int, QString>
 bool QgsLayoutItemAttributeTable::getTableContents( QgsLayoutTableContents &contents )
 {
   contents.clear();
+  mLayerCache.clear();
 
   QgsVectorLayer *layer = sourceLayer();
   if ( !layer )
@@ -396,42 +401,81 @@ bool QgsLayoutItemAttributeTable::getTableContents( QgsLayoutTableContents &cont
     return false;
   }
 
+  const QgsConditionalLayerStyles *conditionalStyles = layer->conditionalStyles();
+
   QgsExpressionContext context = createExpressionContext();
   context.setFields( layer->fields() );
 
   QgsFeatureRequest req;
+  req.setExpressionContext( context );
 
   //prepare filter expression
   std::unique_ptr<QgsExpression> filterExpression;
   bool activeFilter = false;
   if ( mFilterFeatures && !mFeatureFilter.isEmpty() )
   {
-    filterExpression = qgis::make_unique< QgsExpression >( mFeatureFilter );
+    filterExpression = std::make_unique< QgsExpression >( mFeatureFilter );
     if ( !filterExpression->hasParserError() )
     {
       activeFilter = true;
       req.setFilterExpression( mFeatureFilter );
-      req.setExpressionContext( context );
     }
   }
 
+#ifdef HAVE_SERVER_PYTHON_PLUGINS
+  if ( mLayout->renderContext().featureFilterProvider() )
+  {
+    mLayout->renderContext().featureFilterProvider()->filterFeatures( layer, req );
+  }
+#endif
+
   QgsRectangle selectionRect;
+  QgsGeometry visibleRegion;
+  std::unique_ptr< QgsGeometryEngine > visibleMapEngine;
   if ( mMap && mShowOnlyVisibleFeatures )
   {
-    selectionRect = mMap->extent();
-    if ( layer )
+    visibleRegion = QgsGeometry::fromQPolygonF( mMap->visibleExtentPolygon() );
+    selectionRect = visibleRegion.boundingBox();
+    //transform back to layer CRS
+    const QgsCoordinateTransform coordTransform( layer->crs(), mMap->crs(), mLayout->project() );
+    QgsCoordinateTransform extentTransform = coordTransform;
+    extentTransform.setBallparkTransformsAreAppropriate( true );
+    try
     {
-      //transform back to layer CRS
-      QgsCoordinateTransform coordTransform( layer->crs(), mMap->crs(), mLayout->project() );
-      try
+      selectionRect = extentTransform.transformBoundingBox( selectionRect, Qgis::TransformDirection::Reverse );
+      visibleRegion.transform( coordTransform, Qgis::TransformDirection::Reverse );
+    }
+    catch ( QgsCsException &cse )
+    {
+      Q_UNUSED( cse )
+      return false;
+    }
+    visibleMapEngine.reset( QgsGeometry::createGeometryEngine( visibleRegion.constGet() ) );
+    visibleMapEngine->prepareGeometry();
+  }
+
+  QgsGeometry atlasGeometry;
+  std::unique_ptr< QgsGeometryEngine > atlasGeometryEngine;
+  if ( mFilterToAtlasIntersection )
+  {
+    atlasGeometry = mLayout->reportContext().currentGeometry( layer->crs() );
+    if ( !atlasGeometry.isNull() )
+    {
+      if ( selectionRect.isNull() )
       {
-        selectionRect = coordTransform.transformBoundingBox( selectionRect, QgsCoordinateTransform::ReverseTransform );
+        selectionRect = atlasGeometry.boundingBox();
       }
-      catch ( QgsCsException &cse )
+      else
       {
-        Q_UNUSED( cse );
-        return false;
+        selectionRect = selectionRect.intersect( atlasGeometry.boundingBox() );
       }
+
+      atlasGeometryEngine.reset( QgsGeometry::createGeometryEngine( atlasGeometry.constGet() ) );
+      atlasGeometryEngine->prepareGeometry();
+    }
+    else
+    {
+      return false;
     }
   }
 
@@ -442,10 +486,10 @@ bool QgsLayoutItemAttributeTable::getTableContents( QgsLayoutTableContents &cont
     req = relation.getRelatedFeaturesRequest( atlasFeature );
   }
 
-  if ( !selectionRect.isEmpty() )
+  if ( !selectionRect.isNull() )
     req.setFilterRect( selectionRect );
 
-  req.setFlags( mShowOnlyVisibleFeatures ? QgsFeatureRequest::ExactIntersect : QgsFeatureRequest::NoFlags );
+  req.setFlags( mShowOnlyVisibleFeatures ? Qgis::FeatureRequestFlag::ExactIntersect : Qgis::FeatureRequestFlag::NoFlags );
 
   if ( mSource == QgsLayoutItemAttributeTable::AtlasFeature )
   {
@@ -454,10 +498,21 @@ bool QgsLayoutItemAttributeTable::getTableContents( QgsLayoutTableContents &cont
     req.setFilterFid( atlasFeature.id() );
   }
 
+  for ( const QgsLayoutTableColumn &column : std::as_const( mSortColumns ) )
+  {
+    req.addOrderBy( column.attribute(), column.sortOrder() == Qt::AscendingOrder );
+  }
+
   QgsFeature f;
   int counter = 0;
   QgsFeatureIterator fit = layer->getFeatures( req );
 
+  mConditionalStyles.clear();
+  mFeatures.clear();
+
+  QVector< QVector< Cell > > tempContents;
+  QgsLayoutTableContents existingContents;
+
   while ( fit.nextFeature( f ) && counter < mMaximumNumberOfFeatures )
   {
     context.setFeature( f );
@@ -471,63 +526,198 @@ bool QgsLayoutItemAttributeTable::getTableContents( QgsLayoutTableContents &cont
         continue;
       }
     }
+
+    // check against exact map bounds
+    if ( visibleMapEngine )
+    {
+      if ( !f.hasGeometry() )
+        continue;
+
+      if ( !visibleMapEngine->intersects( f.geometry().constGet() ) )
+        continue;
+    }
+
     //check against atlas feature intersection
-    if ( mFilterToAtlasIntersection )
+    if ( atlasGeometryEngine )
     {
       if ( !f.hasGeometry() )
       {
         continue;
       }
-      QgsFeature atlasFeature = mLayout->reportContext().feature();
-      if ( !atlasFeature.hasGeometry() ||
-           !f.geometry().intersects( atlasFeature.geometry() ) )
-      {
-        //feature falls outside current atlas feature
+
+      if ( !atlasGeometryEngine->intersects( f.geometry().constGet() ) )
         continue;
-      }
     }
 
-    QgsLayoutTableRow currentRow;
+    QgsConditionalStyle rowStyle;
 
-    for ( QgsLayoutTableColumn *column : qgis::as_const( mColumns ) )
+    if ( mUseConditionalStyling )
     {
-      int idx = layer->fields().lookupField( column->attribute() );
+      const QList<QgsConditionalStyle> styles = QgsConditionalStyle::matchingConditionalStyles( conditionalStyles->rowStyles(), QVariant(),  context );
+      rowStyle = QgsConditionalStyle::compressStyles( styles );
+    }
+
+    // We need to build up two different lists here -- one is a pair of the cell contents along with the cell style.
+    // We need this one because we do a sorting step later, and we need to ensure that the cell styling is attached to the right row and sorted
+    // correctly when this occurs
+    // We also need a list of just the cell contents, so that we can do a quick check for row uniqueness (when the
+    // corresponding option is enabled)
+    QVector< Cell > currentRow;
+#ifdef HAVE_SERVER_PYTHON_PLUGINS
+    mColumns = filteredColumns();
+#endif
+    currentRow.reserve( mColumns.count() );
+    QgsLayoutTableRow rowContents;
+    rowContents.reserve( mColumns.count() );
+
+    for ( const QgsLayoutTableColumn &column : std::as_const( mColumns ) )
+    {
+      QgsConditionalStyle style;
+      int idx = layer->fields().lookupField( column.attribute() );
       if ( idx != -1 )
       {
-        currentRow << replaceWrapChar( f.attributes().at( idx ) );
+        QVariant val = f.attributes().at( idx );
+
+        if ( mUseConditionalStyling )
+        {
+          QList<QgsConditionalStyle> styles = conditionalStyles->fieldStyles( layer->fields().at( idx ).name() );
+          styles = QgsConditionalStyle::matchingConditionalStyles( styles, val, context );
+          styles.insert( 0, rowStyle );
+          style = QgsConditionalStyle::compressStyles( styles );
+        }
+
+        const QgsEditorWidgetSetup setup = layer->fields().at( idx ).editorWidgetSetup();
+
+        if ( ! setup.isNull() )
+        {
+          QgsFieldFormatter *fieldFormatter = QgsApplication::fieldFormatterRegistry()->fieldFormatter( setup.type() );
+          QVariant cache;
+
+          auto it = mLayerCache.constFind( column.attribute() );
+          if ( it != mLayerCache.constEnd() )
+          {
+            cache = it.value();
+          }
+          else
+          {
+            cache = fieldFormatter->createCache( layer, idx, setup.config() );
+            mLayerCache.insert( column.attribute(), cache );
+          }
+
+          val = fieldFormatter->representValue( layer, idx, setup.config(), cache, val );
+        }
+
+        QVariant v = QgsVariantUtils::isNull( val ) ? QString() : replaceWrapChar( val );
+        currentRow << Cell( v, style, f );
+        rowContents << v;
       }
       else
       {
         // Lets assume it's an expression
-        std::unique_ptr< QgsExpression > expression = qgis::make_unique< QgsExpression >( column->attribute() );
+        auto expression = std::make_unique< QgsExpression >( column.attribute() );
         context.lastScope()->addVariable( QgsExpressionContextScope::StaticVariable( QStringLiteral( "row_number" ), counter + 1, true ) );
         expression->prepare( &context );
         QVariant value = expression->evaluate( &context );
-        currentRow << value;
+
+        currentRow << Cell( value, rowStyle, f );
+        rowContents << value;
       }
     }
 
-    if ( !mShowUniqueRowsOnly || !contentsContainsRow( contents, currentRow ) )
+    if ( mShowUniqueRowsOnly )
     {
-      contents << currentRow;
-      ++counter;
+      if ( contentsContainsRow( existingContents, rowContents ) )
+        continue;
     }
+
+    tempContents << currentRow;
+    existingContents << rowContents;
+    ++counter;
   }
 
-  //sort the list, starting with the last attribute
-  QgsLayoutAttributeTableCompare c;
-  QVector< QPair<int, bool> > sortColumns = sortAttributes();
-  for ( int i = sortColumns.size() - 1; i >= 0; --i )
+  // build final table contents
+  contents.reserve( tempContents.size() );
+  mConditionalStyles.reserve( tempContents.size() );
+  mFeatures.reserve( tempContents.size() );
+  for ( auto it = tempContents.constBegin(); it != tempContents.constEnd(); ++it )
   {
-    c.setSortColumn( sortColumns.at( i ).first );
-    c.setAscending( sortColumns.at( i ).second );
-    std::stable_sort( contents.begin(), contents.end(), c );
+    QgsLayoutTableRow row;
+    QList< QgsConditionalStyle > rowStyles;
+    row.reserve( it->size() );
+    rowStyles.reserve( it->size() );
+
+    for ( auto cellIt = it->constBegin(); cellIt != it->constEnd(); ++cellIt )
+    {
+      row << cellIt->content;
+      rowStyles << cellIt->style;
+      if ( cellIt == it->constBegin() )
+        mFeatures << cellIt->feature;
+    }
+    contents << row;
+    mConditionalStyles << rowStyles;
   }
 
   recalculateTableSize();
   return true;
 }
 
+QgsConditionalStyle QgsLayoutItemAttributeTable::conditionalCellStyle( int row, int column ) const
+{
+  if ( row >= mConditionalStyles.size() )
+    return QgsConditionalStyle();
+
+  return mConditionalStyles.at( row ).at( column );
+}
+
+QgsTextFormat QgsLayoutItemAttributeTable::textFormatForCell( int row, int column ) const
+{
+  QgsTextFormat format = mContentTextFormat;
+
+  const QgsConditionalStyle style = conditionalCellStyle( row, column );
+  if ( style.isValid() )
+  {
+    // apply conditional style formatting to text format
+    const QFont styleFont = style.font();
+    if ( styleFont != QFont() )
+    {
+      QFont newFont = format.font();
+      // we want to keep all the other font settings, like word/letter spacing
+      QgsFontUtils::setFontFamily( newFont, styleFont.family() );
+
+      // warning -- there's a potential trap here! We can't just read QFont::styleName(), as that may be blank even when
+      // the font has the bold or italic attributes set! Reading the style name via QFontInfo avoids this and always returns
+      // a correct style name
+      const QString styleName = QgsFontUtils::resolveFontStyleName( styleFont );
+      if ( !styleName.isEmpty() )
+        newFont.setStyleName( styleName );
+
+      newFont.setStrikeOut( styleFont.strikeOut() );
+      newFont.setUnderline( styleFont.underline() );
+      format.setFont( newFont );
+      if ( styleName.isEmpty() )
+      {
+        // we couldn't find a direct match for the conditional font's bold/italic settings as a font style name.
+        // This means the conditional style is using Qt's "faux bold/italic" mode. Even though it causes reduced quality font
+        // rendering, we'll apply it here anyway just to ensure that the rendered font styling matches the conditional style.
+        if ( styleFont.bold() )
+          format.setForcedBold( true );
+        if ( styleFont.italic() )
+          format.setForcedItalic( true );
+      }
+    }
+  }
+
+  return format;
+}
+
+QgsExpressionContextScope *QgsLayoutItemAttributeTable::scopeForCell( int row, int column ) const
+{
+  std::unique_ptr< QgsExpressionContextScope >scope( QgsLayoutTable::scopeForCell( row, column ) );
+  scope->setFeature( mFeatures.value( row ) );
+  scope->setFields( scope->feature().fields() );
+  return scope.release();
+}
+
 QgsExpressionContext QgsLayoutItemAttributeTable::createExpressionContext() const
 {
   QgsExpressionContext context = QgsLayoutTable::createExpressionContext();
@@ -550,6 +740,7 @@ void QgsLayoutItemAttributeTable::finalizeRestoreFromXml()
     {
       //if we have found a valid map item, listen out to extent changes on it and refresh the table
       connect( mMap, &QgsLayoutItemMap::extentChanged, this, &QgsLayoutTable::refreshAttributes );
+      connect( mMap, &QgsLayoutItemMap::mapRotationChanged, this, &QgsLayoutTable::refreshAttributes );
     }
   }
 }
@@ -559,7 +750,7 @@ void QgsLayoutItemAttributeTable::refreshDataDefinedProperty( const QgsLayoutObj
   QgsExpressionContext context = createExpressionContext();
 
   if ( mSource == QgsLayoutItemAttributeTable::LayerAttributes &&
-       ( property == QgsLayoutObject::AttributeTableSourceLayer || property == QgsLayoutObject::AllProperties ) )
+       ( property == QgsLayoutObject::DataDefinedProperty::AttributeTableSourceLayer || property == QgsLayoutObject::DataDefinedProperty::AllProperties ) )
   {
     mDataDefinedVectorLayer = nullptr;
 
@@ -567,7 +758,7 @@ void QgsLayoutItemAttributeTable::refreshDataDefinedProperty( const QgsLayoutObj
     if ( QgsVectorLayer *currentLayer = mVectorLayer.get() )
       currentLayerIdentifier = currentLayer->id();
 
-    const QString layerIdentifier = mDataDefinedProperties.valueAsString( QgsLayoutObject::AttributeTableSourceLayer, context, currentLayerIdentifier );
+    const QString layerIdentifier = mDataDefinedProperties.valueAsString( QgsLayoutObject::DataDefinedProperty::AttributeTableSourceLayer, context, currentLayerIdentifier );
     QgsVectorLayer *ddLayer = qobject_cast< QgsVectorLayer * >( QgsLayoutUtils::mapLayerFromString( layerIdentifier, mLayout->project() ) );
     if ( ddLayer )
       mDataDefinedVectorLayer = ddLayer;
@@ -587,6 +778,65 @@ QVariant QgsLayoutItemAttributeTable::replaceWrapChar( const QVariant &variant )
   return replaced;
 }
 
+#ifdef HAVE_SERVER_PYTHON_PLUGINS
+QgsLayoutTableColumns QgsLayoutItemAttributeTable::filteredColumns()
+{
+
+  QgsLayoutTableColumns allowedColumns { mColumns };
+
+  // Filter columns
+  if ( mLayout->renderContext().featureFilterProvider() )
+  {
+
+    QgsVectorLayer *source { sourceLayer() };
+
+    if ( ! source )
+    {
+      return allowedColumns;
+    }
+
+    QHash<const QString, QSet<QString>> columnAttributesMap;
+    QSet<QString> allowedAttributes;
+
+    for ( const auto &c : std::as_const( allowedColumns ) )
+    {
+      if ( ! c.attribute().isEmpty() && ! columnAttributesMap.contains( c.attribute() ) )
+      {
+        columnAttributesMap[ c.attribute() ] = QSet<QString>();
+        const QgsExpression columnExp { c.attribute() };
+        const auto constRefs { columnExp.findNodes<QgsExpressionNodeColumnRef>() };
+        for ( const auto &cref : constRefs )
+        {
+          columnAttributesMap[ c.attribute() ].insert( cref->name() );
+          allowedAttributes.insert( cref->name() );
+        }
+      }
+    }
+
+    const QStringList filteredAttributes { layout()->renderContext().featureFilterProvider()->layerAttributes( source, allowedAttributes.values() ) };
+    const QSet<QString> filteredAttributesSet( filteredAttributes.constBegin(), filteredAttributes.constEnd() );
+    if ( filteredAttributesSet != allowedAttributes )
+    {
+      const auto forbidden { allowedAttributes.subtract( filteredAttributesSet ) };
+      allowedColumns.erase( std::remove_if( allowedColumns.begin(), allowedColumns.end(), [ &columnAttributesMap, &forbidden ]( QgsLayoutTableColumn & c ) -> bool
+      {
+        for ( const auto &f : std::as_const( forbidden ) )
+        {
+          if ( columnAttributesMap[ c.attribute() ].contains( f ) )
+          {
+            return true;
+          }
+        }
+        return false;
+      } ), allowedColumns.end() );
+
+    }
+  }
+
+  return allowedColumns;
+}
+#endif
+
 QgsVectorLayer *QgsLayoutItemAttributeTable::sourceLayer() const
 {
   switch ( mSource )
@@ -617,44 +867,11 @@ void QgsLayoutItemAttributeTable::removeLayer( const QString &layerId )
     {
       mVectorLayer.setLayer( nullptr );
       //remove existing columns
-      qDeleteAll( mColumns );
       mColumns.clear();
     }
   }
 }
 
-static bool columnsBySortRank( QPair<int, QgsLayoutTableColumn * > a, QPair<int, QgsLayoutTableColumn * > b )
-{
-  return a.second->sortByRank() < b.second->sortByRank();
-}
-
-QVector<QPair<int, bool> > QgsLayoutItemAttributeTable::sortAttributes() const
-{
-  //generate list of all sorted columns
-  QVector< QPair<int, QgsLayoutTableColumn * > > sortedColumns;
-  int idx = 0;
-  for ( QgsLayoutTableColumn *column : mColumns )
-  {
-    if ( column->sortByRank() > 0 )
-    {
-      sortedColumns.append( qMakePair( idx, column ) );
-    }
-    idx++;
-  }
-
-  //sort columns by rank
-  std::sort( sortedColumns.begin(), sortedColumns.end(), columnsBySortRank );
-
-  //generate list of column index, bool for sort direction (to match 2.0 api)
-  QVector<QPair<int, bool> > attributesBySortRank;
-  for ( auto &column : qgis::as_const( sortedColumns ) )
-  {
-    attributesBySortRank.append( qMakePair( column.first,
-                                            column.second->sortOrder() == Qt::AscendingOrder ) );
-  }
-  return attributesBySortRank;
-}
-
 void QgsLayoutItemAttributeTable::setWrapString( const QString &wrapString )
 {
   if ( wrapString == mWrapString )
@@ -681,6 +898,7 @@ bool QgsLayoutItemAttributeTable::writePropertiesToElement( QDomElement &tableEl
   tableElem.setAttribute( QStringLiteral( "filterFeatures" ), mFilterFeatures ? QStringLiteral( "true" ) : QStringLiteral( "false" ) );
   tableElem.setAttribute( QStringLiteral( "featureFilter" ), mFeatureFilter );
   tableElem.setAttribute( QStringLiteral( "wrapString" ), mWrapString );
+  tableElem.setAttribute( QStringLiteral( "useConditionalStyling" ), mUseConditionalStyling );
 
   if ( mMap )
   {
@@ -699,8 +917,7 @@ bool QgsLayoutItemAttributeTable::writePropertiesToElement( QDomElement &tableEl
 
 bool QgsLayoutItemAttributeTable::readPropertiesFromElement( const QDomElement &itemElem, const QDomDocument &doc, const QgsReadWriteContext &context )
 {
-  QgsVectorLayer *prevLayer = sourceLayer();
-  if ( prevLayer )
+  if ( QgsVectorLayer *prevLayer = sourceLayer() )
   {
     //disconnect from previous layer
     disconnect( prevLayer, &QgsVectorLayer::layerModified, this, &QgsLayoutTable::refreshAttributes );
@@ -724,12 +941,14 @@ bool QgsLayoutItemAttributeTable::readPropertiesFromElement( const QDomElement &
   mFeatureFilter = itemElem.attribute( QStringLiteral( "featureFilter" ), QString() );
   mMaximumNumberOfFeatures = itemElem.attribute( QStringLiteral( "maxFeatures" ), QStringLiteral( "5" ) ).toInt();
   mWrapString = itemElem.attribute( QStringLiteral( "wrapString" ) );
+  mUseConditionalStyling = itemElem.attribute( QStringLiteral( "useConditionalStyling" ), QStringLiteral( "0" ) ).toInt();
 
   //map
   mMapUuid = itemElem.attribute( QStringLiteral( "mapUuid" ) );
   if ( mMap )
   {
     disconnect( mMap, &QgsLayoutItemMap::extentChanged, this, &QgsLayoutTable::refreshAttributes );
+    disconnect( mMap, &QgsLayoutItemMap::mapRotationChanged, this, &QgsLayoutTable::refreshAttributes );
     mMap = nullptr;
   }
   // setting new mMap occurs in finalizeRestoreFromXml
@@ -743,7 +962,8 @@ bool QgsLayoutItemAttributeTable::readPropertiesFromElement( const QDomElement &
   mVectorLayer.resolveWeakly( mLayout->project() );
 
   //connect to new layer
-  connect( sourceLayer(), &QgsVectorLayer::layerModified, this, &QgsLayoutTable::refreshAttributes );
+  if ( QgsVectorLayer *newLayer = sourceLayer() )
+    connect( newLayer, &QgsVectorLayer::layerModified, this, &QgsLayoutTable::refreshAttributes );
 
   refreshAttributes();
 
