@@ -896,7 +896,7 @@ QgsLineSymbol *QgsMapInfoSymbolConverter::convertLineSymbol( int identifier, Qgs
   else if ( identifier >= 41 && identifier < 45 )
   {
     const int count = identifier - 40;
-    QgsSimpleLineSymbolLayer *simpleLine = dynamic_cast< QgsSimpleLineSymbolLayer * >( symbol->symbolLayer( 0 ) );
+    QgsSimpleLineSymbolLayer *simpleLine = qgis::down_cast< QgsSimpleLineSymbolLayer * >( symbol->symbolLayer( 0 ) );
     simpleLine->setCustomDashVector( QVector< qreal >() <<  0 << 5.25 * size << 4 * size << ( 3.25 * size  + ( count - 1 ) * ( 7.25 * size ) ) );
     simpleLine->setCustomDashPatternUnit( sizeUnit );
     simpleLine->setUseCustomDashPattern( true );
@@ -1398,7 +1398,7 @@ QgsMarkerSymbol *QgsMapInfoSymbolConverter::convertMarkerSymbol( int identifier,
   bool isNull = false;
   bool hasShadow = false;
   double angle = 0;
-  QgsMarkerSymbolLayer::VerticalAnchorPoint vertAlign = QgsMarkerSymbolLayer::VCenter;
+  Qgis::VerticalAnchorPoint vertAlign = Qgis::VerticalAnchorPoint::Center;
   QPointF shadowOffset;
   switch ( identifier )
   {
@@ -1486,14 +1486,14 @@ QgsMarkerSymbol *QgsMapInfoSymbolConverter::convertMarkerSymbol( int identifier,
       shape = Qgis::MarkerShape::Arrow;
       size *= 0.66666;
       angle = 45;
-      vertAlign = QgsMarkerSymbolLayer::Top;
+      vertAlign = Qgis::VerticalAnchorPoint::Top;
       break;
 
     case 48:
       shape = Qgis::MarkerShape::Arrow;
       size *= 0.66666;
       angle = 225;
-      vertAlign = QgsMarkerSymbolLayer::Top;
+      vertAlign = Qgis::VerticalAnchorPoint::Top;
       break;
 
     case 49:
