@@ -19,26 +19,33 @@
 #include "ui_qgsvaluerelationconfigdlgbase.h"
 
 #include "qgseditorconfigwidget.h"
+#include "qgis_gui.h"
 
-/** \ingroup gui
+SIP_NO_FILE
+
+/**
+ * \ingroup gui
  * \class QgsValueRelationConfigDlg
+ * \brief Configuration widget for value relation widgets.
  * \note not available in Python bindings
  */
-
 class GUI_EXPORT QgsValueRelationConfigDlg : public QgsEditorConfigWidget, private Ui::QgsValueRelationConfigDlgBase
 {
     Q_OBJECT
 
   public:
-    explicit QgsValueRelationConfigDlg( QgsVectorLayer* vl, int fieldIdx, QWidget *parent = nullptr );
+    explicit QgsValueRelationConfigDlg( QgsVectorLayer *vl, int fieldIdx, QWidget *parent = nullptr );
 
   public slots:
     void editExpression();
 
     // QgsEditorConfigWidget interface
   public:
-    QgsEditorWidgetConfig config() override;
-    void setConfig( const QgsEditorWidgetConfig& config ) override;
+    QVariantMap config() override;
+    void setConfig( const QVariantMap &config ) override;
+
+  private slots:
+    void layerChanged();
 };
 
 #endif // QGSVALUERELATIONCONFIGDLG_H
