@@ -17,31 +17,27 @@
 #include <QLabel>
 #include <QLineEdit>
 #include <QPushButton>
-#include <QRegExpValidator>
+#include <QRegularExpressionValidator>
 #include <QSizePolicy>
+#include <QCompleter>
 
 #include "qgslogger.h"
 #include "qgsnewnamedialog.h"
+#include "moc_qgsnewnamedialog.cpp"
 
-QgsNewNameDialog::QgsNewNameDialog( const QString& source, const QString& initial,
-                                    const QStringList& extensions, const QStringList& existing,
-                                    const QRegExp& regexp, Qt::CaseSensitivity cs,
-                                    QWidget *parent, const Qt::WindowFlags& flags )
-    : QgsDialog( parent, flags, QDialogButtonBox::Ok | QDialogButtonBox::Cancel )
-    , mExiting( existing )
-    , mExtensions( extensions )
-    , mCaseSensitivity( cs )
-    , mNamesLabel( nullptr )
-    , mRegexp( regexp )
-    , mOverwriteEnabled( true )
+QgsNewNameDialog::QgsNewNameDialog( const QString &source, const QString &initial, const QStringList &extensions, const QStringList &existing, Qt::CaseSensitivity cs, QWidget *parent, Qt::WindowFlags flags )
+  : QgsDialog( parent, flags, QDialogButtonBox::Ok | QDialogButtonBox::Cancel )
+  , mExiting( existing )
+  , mExtensions( extensions )
+  , mCaseSensitivity( cs )
 {
-  setWindowTitle( tr( "New name" ) );
-  QDialog::layout()->setSizeConstraint( QLayout::SetMinimumSize );
+  setWindowTitle( tr( "New Name" ) );
+  QgsDialog::layout()->setSizeConstraint( QLayout::SetMinimumSize );
   layout()->setSizeConstraint( QLayout::SetMinimumSize );
   layout()->setSpacing( 6 );
   mOkString = buttonBox()->button( QDialogButtonBox::Ok )->text();
   QString hintString;
-  QString nameDesc = mExtensions.isEmpty() ? tr( "name" ) : tr( "base name" );
+  const QString nameDesc = mExtensions.isEmpty() ? tr( "name" ) : tr( "base name" );
   if ( source.isEmpty() )
   {
     hintString = tr( "Enter new %1" ).arg( nameDesc );
@@ -54,16 +50,13 @@ QgsNewNameDialog::QgsNewNameDialog( const QString& source, const QString& initia
   layout()->addWidget( mHintLabel );
 
   mLineEdit = new QLineEdit( initial, this );
-  if ( !regexp.isEmpty() )
-  {
-    QRegExpValidator *validator = new QRegExpValidator( regexp, this );
-    mLineEdit->setValidator( validator );
-  }
-  mLineEdit->setMinimumWidth( mLineEdit->fontMetrics().width( "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx" ) );
-  connect( mLineEdit, SIGNAL( textChanged( QString ) ), this, SLOT( nameChanged() ) );
+  mLineEdit->setMinimumWidth( mLineEdit->fontMetrics().horizontalAdvance( 'x' ) * 44 );
+
+  connect( mLineEdit, &QLineEdit::textChanged, this, &QgsNewNameDialog::nameChanged );
+  connect( mLineEdit, &QLineEdit::textChanged, this, &QgsNewNameDialog::newNameChanged );
   layout()->addWidget( mLineEdit );
 
-  mNamesLabel = new QLabel( " ", this );
+  mNamesLabel = new QLabel( QStringLiteral( " " ), this );
   mNamesLabel->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
   if ( !mExtensions.isEmpty() )
   {
@@ -71,7 +64,7 @@ QgsNewNameDialog::QgsNewNameDialog( const QString& source, const QString& initia
     layout()->addWidget( mNamesLabel );
   }
 
-  mErrorLabel = new QLabel( " ", this );
+  mErrorLabel = new QLabel( QStringLiteral( " " ), this );
   mErrorLabel->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
   mErrorLabel->setWordWrap( true );
   layout()->addWidget( mErrorLabel );
@@ -98,52 +91,87 @@ void QgsNewNameDialog::setOverwriteEnabled( bool enabled )
   nameChanged(); //update UI
 }
 
-void QgsNewNameDialog::setConflictingNameWarning( const QString& string )
+void QgsNewNameDialog::setAllowEmptyName( bool allowed )
+{
+  mAllowEmptyName = allowed;
+  nameChanged(); //update UI
+}
+
+void QgsNewNameDialog::setConflictingNameWarning( const QString &string )
 {
   mConflictingNameWarning = string;
   nameChanged(); //update UI
 }
 
-QString QgsNewNameDialog::highlightText( const QString& text )
+void QgsNewNameDialog::setRegularExpression( const QString &expression )
+{
+  if ( !expression.isEmpty() )
+  {
+    mRegularExpression = QRegularExpression( QRegularExpression::anchoredPattern( expression ) );
+    QRegularExpressionValidator *validator = new QRegularExpressionValidator( mRegularExpression, this );
+    mLineEdit->setValidator( validator );
+  }
+  else
+  {
+    mRegularExpression = QRegularExpression();
+    mLineEdit->setValidator( nullptr );
+  }
+  nameChanged();
+}
+
+void QgsNewNameDialog::setShowExistingNamesCompleter( bool show )
+{
+  if ( !show )
+  {
+    mLineEdit->setCompleter( nullptr );
+    return;
+  }
+
+  QCompleter *completer = new QCompleter( mExiting, this );
+  completer->setCaseSensitivity( Qt::CaseInsensitive );
+  completer->setCompletionMode( QCompleter::UnfilteredPopupCompletion );
+  mLineEdit->setCompleter( completer );
+}
+
+QString QgsNewNameDialog::highlightText( const QString &text )
 {
   return "<b>" + text + "</b>";
 }
 
 void QgsNewNameDialog::nameChanged()
 {
-
   QString namesString = tr( "Full names" ) + ": ";
   if ( !mExtensions.isEmpty() )
   {
     mNamesLabel->setText( namesString );
   }
-  mErrorLabel->setText( " " ); // space to keep vertical space
-  QPushButton* okButton = buttonBox()->button( QDialogButtonBox::Ok );
+  mErrorLabel->setText( QStringLiteral( " " ) ); // space to keep vertical space
+  QPushButton *okButton = buttonBox()->button( QDialogButtonBox::Ok );
   okButton->setText( mOkString );
   okButton->setEnabled( true );
 
-  QString newName = name();
+  const QString newName = name();
 
-  if ( newName.length() == 0 || ( !mRegexp.isEmpty() && !mRegexp.exactMatch( newName ) ) )
+  if ( newName.length() == 0 || ( !mRegularExpression.pattern().isEmpty() && !mRegularExpression.match( newName ).hasMatch() ) )
   {
     //mErrorLabel->setText( highlightText( tr( "Enter new name" ) );
-    okButton->setEnabled( false );
+    okButton->setEnabled( mAllowEmptyName );
     return;
   }
 
-  QStringList newNames = fullNames( newName, mExtensions );
+  const QStringList newNames = fullNames( newName, mExtensions );
   if ( !mExtensions.isEmpty() )
   {
-    namesString += ' ' + newNames.join( ", " );
+    namesString += ' ' + newNames.join( QLatin1String( ", " ) );
     mNamesLabel->setText( namesString );
   }
 
-  QStringList conflicts = matching( newNames, mExiting, mCaseSensitivity );
+  const QStringList conflicts = matching( newNames, mExiting, mCaseSensitivity );
 
   if ( !conflicts.isEmpty() )
   {
-    QString warning = !mConflictingNameWarning.isEmpty() ? mConflictingNameWarning
-                      : tr( "%n Name(s) %1 exists", nullptr, conflicts.size() ).arg( conflicts.join( ", " ) );
+    const QString warning = !mConflictingNameWarning.isEmpty() ? mConflictingNameWarning
+                                                               : tr( "%n Name(s) %1 exists", nullptr, conflicts.size() ).arg( conflicts.join( QLatin1String( ", " ) ) );
     mErrorLabel->setText( highlightText( warning ) );
     if ( mOverwriteEnabled )
     {
@@ -162,13 +190,13 @@ QString QgsNewNameDialog::name() const
   return mLineEdit->text().trimmed();
 }
 
-QStringList QgsNewNameDialog::fullNames( const QString& name, const QStringList& extensions )
+QStringList QgsNewNameDialog::fullNames( const QString &name, const QStringList &extensions )
 {
   QStringList list;
-  Q_FOREACH ( const QString& ext, extensions )
+  const auto constExtensions = extensions;
+  for ( const QString &ext : constExtensions )
   {
     list << name + ext;
-
   }
   if ( list.isEmpty() )
   {
@@ -177,14 +205,15 @@ QStringList QgsNewNameDialog::fullNames( const QString& name, const QStringList&
   return list;
 }
 
-QStringList QgsNewNameDialog::matching( const QStringList& newNames, const QStringList& existingNames,
-                                        Qt::CaseSensitivity cs )
+QStringList QgsNewNameDialog::matching( const QStringList &newNames, const QStringList &existingNames, Qt::CaseSensitivity cs )
 {
   QStringList list;
 
-  Q_FOREACH ( const QString& newName, newNames )
+  const auto constNewNames = newNames;
+  for ( const QString &newName : constNewNames )
   {
-    Q_FOREACH ( const QString& existingName, existingNames )
+    const auto constExistingNames = existingNames;
+    for ( const QString &existingName : constExistingNames )
     {
       if ( existingName.compare( newName, cs ) == 0 )
       {
@@ -195,10 +224,9 @@ QStringList QgsNewNameDialog::matching( const QStringList& newNames, const QStri
   return list;
 }
 
-bool QgsNewNameDialog::exists( const QString& name, const QStringList& extensions,
-                               const QStringList& existing, Qt::CaseSensitivity cs )
+bool QgsNewNameDialog::exists( const QString &name, const QStringList &extensions, const QStringList &existing, Qt::CaseSensitivity cs )
 {
-  QStringList newNames = fullNames( name, extensions );
-  QStringList conflicts = matching( newNames, existing, cs );
+  const QStringList newNames = fullNames( name, extensions );
+  const QStringList conflicts = matching( newNames, existing, cs );
   return !conflicts.isEmpty();
 }
