@@ -22,11 +22,12 @@
 #include "qgsfeature.h"
 #include "qgsfeatureiterator.h"
 #include "qgsgeometry.h"
-#include "qgsproviderregistry.h"
 #include "qgsrasterdataprovider.h"
 #include "qgsrasteriterator.h"
-
 #include "qgsgrassimport.h"
+#include "moc_qgsgrassimport.cpp"
+
+#include <QFileInfo>
 
 extern "C"
 {
@@ -39,30 +40,30 @@ extern "C"
 //------------------------------ QgsGrassImport ------------------------------------
 QgsGrassImportIcon *QgsGrassImportIcon::instance()
 {
-  static QgsGrassImportIcon* sInstance = new QgsGrassImportIcon();
+  static QgsGrassImportIcon *sInstance = new QgsGrassImportIcon();
   return sInstance;
 }
 
 QgsGrassImportIcon::QgsGrassImportIcon()
-    : QgsAnimatedIcon( QgsApplication::iconPath( "/mIconImport.gif" ) )
+  : QgsAnimatedIcon( QgsApplication::iconPath( QStringLiteral( "/mIconImport.gif" ) ) )
 {
 }
 
 //------------------------------ QgsGrassImportProcess ------------------------------------
 QgsGrassImportProgress::QgsGrassImportProgress( QProcess *process, QObject *parent )
-    : QObject( parent )
-    , mProcess( process )
-    , mProgressMin( 0 )
-    , mProgressMax( 0 )
-    , mProgressValue( 0 )
+  : QObject( parent )
+  , mProcess( process )
+  , mProgressMin( 0 )
+  , mProgressMax( 0 )
+  , mProgressValue( 0 )
 {
-  connect( mProcess, SIGNAL( readyReadStandardError() ), SLOT( onReadyReadStandardError() ) );
+  connect( mProcess, &QProcess::readyReadStandardError, this, &QgsGrassImportProgress::onReadyReadStandardError );
 }
 
 void QgsGrassImportProgress::setProcess( QProcess *process )
 {
   mProcess = process;
-  connect( mProcess, SIGNAL( readyReadStandardError() ), SLOT( onReadyReadStandardError() ) );
+  connect( mProcess, &QProcess::readyReadStandardError, this, &QgsGrassImportProgress::onReadyReadStandardError );
 }
 
 void QgsGrassImportProgress::onReadyReadStandardError()
@@ -71,12 +72,12 @@ void QgsGrassImportProgress::onReadyReadStandardError()
   {
     // TODO: parse better progress output
     QString output = QString::fromLocal8Bit( mProcess->readAllStandardError() );
-    Q_FOREACH ( const QString& line, output.split( "\n" ) )
+    for ( const QString &line : output.split( "\n" ) )
     {
-      QgsDebugMsg( "line = '" + line + "'" );
+      QgsDebugMsgLevel( "line = '" + line + "'", 3 );
       QString text, html;
       int value;
-      QgsGrass::ModuleOutput type =  QgsGrass::parseModuleOutput( line, text, html, value );
+      QgsGrass::ModuleOutput type = QgsGrass::parseModuleOutput( line, text, html, value );
       if ( type == QgsGrass::OutputPercent )
       {
         mProgressMin = 0;
@@ -97,12 +98,12 @@ void QgsGrassImportProgress::onReadyReadStandardError()
   }
 }
 
-void QgsGrassImportProgress::append( const QString & html )
+void QgsGrassImportProgress::append( const QString &html )
 {
-  QgsDebugMsg( "html = " + html );
+  QgsDebugMsgLevel( "html = " + html, 3 );
   if ( !mProgressHtml.isEmpty() )
   {
-    mProgressHtml += "<br>";
+    mProgressHtml += QLatin1String( "<br>" );
   }
   mProgressHtml += html;
   emit progressChanged( html, mProgressHtml, mProgressMin, mProgressMax, mProgressValue );
@@ -113,43 +114,39 @@ void QgsGrassImportProgress::setRange( int min, int max )
   mProgressMin = min;
   mProgressMax = max;
   mProgressValue = min;
-  emit progressChanged( "", mProgressHtml, mProgressMin, mProgressMax, mProgressValue );
+  emit progressChanged( QString(), mProgressHtml, mProgressMin, mProgressMax, mProgressValue );
 }
 
 void QgsGrassImportProgress::setValue( int value )
 {
   mProgressValue = value;
-  emit progressChanged( "", mProgressHtml, mProgressMin, mProgressMax, mProgressValue );
+  emit progressChanged( QString(), mProgressHtml, mProgressMin, mProgressMax, mProgressValue );
 }
 
 //------------------------------ QgsGrassImport ------------------------------------
-QgsGrassImport::QgsGrassImport( QgsGrassObject grassObject )
-    : QObject()
-    , mGrassObject( grassObject )
-    , mCanceled( false )
-    , mProcess( 0 )
-    , mProgress( 0 )
-    , mFutureWatcher( 0 )
+QgsGrassImport::QgsGrassImport( const QgsGrassObject &grassObject )
+  : mGrassObject( grassObject )
+  , mCanceled( false )
 {
   // QMovie used by QgsAnimatedIcon is using QTimer which cannot be start from another thread
   // (it works on Linux however) so we cannot start it connecting from QgsGrassImportItem and
   // connect it also here (QgsGrassImport is constructed on the main thread) to a slot doing nothing.
-  QgsGrassImportIcon::instance()->connectFrameChanged( this, SLOT( frameChanged() ) );
+  QgsGrassImportIcon::instance()->connectFrameChanged( this, &QgsGrassImport::frameChanged );
 }
 
 QgsGrassImport::~QgsGrassImport()
 {
   if ( mFutureWatcher && !mFutureWatcher->isFinished() )
   {
-    QgsDebugMsg( "mFutureWatcher not finished -> waitForFinished()" );
+    QgsDebugMsgLevel( "mFutureWatcher not finished -> waitForFinished()", 3 );
     mFutureWatcher->waitForFinished();
   }
-  QgsGrassImportIcon::instance()->disconnectFrameChanged( this, SLOT( frameChanged() ) );
+  QgsGrassImportIcon::instance()->disconnectFrameChanged( this, &QgsGrassImport::frameChanged );
 }
 
-void QgsGrassImport::setError( QString error )
+void QgsGrassImport::setError( const QString &error )
 {
-  QgsDebugMsg( "error: " + error );
+  QgsDebugError( "error: " + error );
   mError = error;
 }
 
@@ -161,7 +158,7 @@ QString QgsGrassImport::error()
 void QgsGrassImport::importInThread()
 {
   mFutureWatcher = new QFutureWatcher<bool>( this );
-  connect( mFutureWatcher, SIGNAL( finished() ), SLOT( onFinished() ) );
+  connect( mFutureWatcher, &QFutureWatcherBase::finished, this, &QgsGrassImport::onFinished );
   mFutureWatcher->setFuture( QtConcurrent::run( run, this ) );
 }
 
@@ -194,13 +191,12 @@ void QgsGrassImport::cancel()
 }
 
 //------------------------------ QgsGrassRasterImport ------------------------------------
-QgsGrassRasterImport::QgsGrassRasterImport( QgsRasterPipe* pipe, const QgsGrassObject& grassObject,
-    const QgsRectangle &extent, int xSize, int ySize )
-    : QgsGrassImport( grassObject )
-    , mPipe( pipe )
-    , mExtent( extent )
-    , mXSize( xSize )
-    , mYSize( ySize )
+QgsGrassRasterImport::QgsGrassRasterImport( std::unique_ptr<QgsRasterPipe> pipe, const QgsGrassObject &grassObject, const QgsRectangle &extent, int xSize, int ySize )
+  : QgsGrassImport( grassObject )
+  , mPipe( std::move( pipe ) )
+  , mExtent( extent )
+  , mXSize( xSize )
+  , mYSize( ySize )
 {
 }
 
@@ -208,30 +204,29 @@ QgsGrassRasterImport::~QgsGrassRasterImport()
 {
   if ( mFutureWatcher && !mFutureWatcher->isFinished() )
   {
-    QgsDebugMsg( "mFutureWatcher not finished -> waitForFinished()" );
+    QgsDebugMsgLevel( "mFutureWatcher not finished -> waitForFinished()", 3 );
     mFutureWatcher->waitForFinished();
   }
-  delete mPipe;
 }
 
 bool QgsGrassRasterImport::import()
 {
   if ( !mPipe )
   {
-    setError( "Pipe is null." );
+    setError( QStringLiteral( "Pipe is null." ) );
     return false;
   }
 
-  QgsRasterDataProvider * provider = mPipe->provider();
+  QgsRasterDataProvider *provider = mPipe->provider();
   if ( !provider )
   {
-    setError( "Pipe has no provider." );
+    setError( QStringLiteral( "Pipe has no provider." ) );
     return false;
   }
 
   if ( !provider->isValid() )
   {
-    setError( "Provider is not valid." );
+    setError( QStringLiteral( "Provider is not valid." ) );
     return false;
   }
 
@@ -239,7 +234,7 @@ bool QgsGrassRasterImport::import()
   struct Cell_head defaultWindow;
   if ( !QgsGrass::defaultRegion( mGrassObject.gisdbase(), mGrassObject.location(), &defaultWindow ) )
   {
-    setError( "Cannot get default window" );
+    setError( QStringLiteral( "Cannot get default window" ) );
     return false;
   }
 
@@ -248,54 +243,55 @@ bool QgsGrassRasterImport::import()
   int blueBand = 0;
   for ( int band = 1; band <= provider->bandCount(); band++ )
   {
-    QgsDebugMsg( QString( "band = %1" ).arg( band ) );
-    int colorInterpretation = provider->colorInterpretation( band );
-    if ( colorInterpretation ==  QgsRaster::RedBand )
+    QgsDebugMsgLevel( QString( "band = %1" ).arg( band ), 3 );
+    Qgis::RasterColorInterpretation colorInterpretation = provider->colorInterpretation( band );
+    if ( colorInterpretation == Qgis::RasterColorInterpretation::RedBand )
     {
       redBand = band;
     }
-    else if ( colorInterpretation ==  QgsRaster::GreenBand )
+    else if ( colorInterpretation == Qgis::RasterColorInterpretation::GreenBand )
     {
       greenBand = band;
     }
-    else if ( colorInterpretation ==  QgsRaster::BlueBand )
+    else if ( colorInterpretation == Qgis::RasterColorInterpretation::BlueBand )
     {
       blueBand = band;
     }
 
-    QGis::DataType qgis_out_type = QGis::UnknownDataType;
+    Qgis::DataType qgis_out_type = Qgis::DataType::UnknownDataType;
 #ifdef QGISDEBUG
     RASTER_MAP_TYPE data_type = -1;
 #endif
     switch ( provider->dataType( band ) )
     {
-      case QGis::Byte:
-      case QGis::UInt16:
-      case QGis::Int16:
-      case QGis::UInt32:
-      case QGis::Int32:
-        qgis_out_type = QGis::Int32;
+      case Qgis::DataType::Byte:
+      case Qgis::DataType::UInt16:
+      case Qgis::DataType::Int8:
+      case Qgis::DataType::Int16:
+      case Qgis::DataType::UInt32:
+      case Qgis::DataType::Int32:
+        qgis_out_type = Qgis::DataType::Int32;
         break;
-      case QGis::Float32:
-        qgis_out_type = QGis::Float32;
+      case Qgis::DataType::Float32:
+        qgis_out_type = Qgis::DataType::Float32;
         break;
-      case QGis::Float64:
-        qgis_out_type = QGis::Float64;
+      case Qgis::DataType::Float64:
+        qgis_out_type = Qgis::DataType::Float64;
         break;
-      case QGis::ARGB32:
-      case QGis::ARGB32_Premultiplied:
-        qgis_out_type = QGis::Int32;  // split to multiple bands?
+      case Qgis::DataType::ARGB32:
+      case Qgis::DataType::ARGB32_Premultiplied:
+        qgis_out_type = Qgis::DataType::Int32; // split to multiple bands?
         break;
-      case QGis::CInt16:
-      case QGis::CInt32:
-      case QGis::CFloat32:
-      case QGis::CFloat64:
-      case QGis::UnknownDataType:
-        setError( tr( "Data type %1 not supported" ).arg( provider->dataType( band ) ) );
+      case Qgis::DataType::CInt16:
+      case Qgis::DataType::CInt32:
+      case Qgis::DataType::CFloat32:
+      case Qgis::DataType::CFloat64:
+      case Qgis::DataType::UnknownDataType:
+        setError( tr( "Data type %1 not supported" ).arg( static_cast<int>( provider->dataType( band ) ) ) );
         return false;
     }
 
-    QgsDebugMsg( QString( "data_type = %1" ).arg( data_type ) );
+    QgsDebugMsgLevel( QString( "data_type = %1" ).arg( data_type ), 3 );
 
     QString module = QgsGrass::qgisGrassModulePath() + "/qgis.r.in";
     QStringList arguments;
@@ -303,9 +299,9 @@ bool QgsGrassRasterImport::import()
     if ( provider->bandCount() > 1 )
     {
       // raster.<band> to keep in sync with r.in.gdal
-      name += QString( ".%1" ).arg( band );
+      name += QStringLiteral( ".%1" ).arg( band );
     }
-    arguments.append( "output=" + name );    // get list of all output names
+    arguments.append( "output=" + name ); // get list of all output names
     QTemporaryFile gisrcFile;
     try
     {
@@ -330,8 +326,8 @@ bool QgsGrassRasterImport::import()
 
     outStream << ( qint32 ) defaultWindow.proj;
     outStream << ( qint32 ) defaultWindow.zone;
-    outStream << mExtent << ( qint32 )mXSize << ( qint32 )mYSize;
-    outStream << ( qint32 )qgis_out_type;
+    outStream << mExtent << ( qint32 ) mXSize << ( qint32 ) mYSize;
+    outStream << ( qint32 ) qgis_out_type;
 
     // calculate reasonable block size (5MB)
     int maximumTileHeight = 5000000 / mXSize;
@@ -352,7 +348,7 @@ bool QgsGrassRasterImport::import()
     int iterTop = 0;
     int iterCols = 0;
     int iterRows = 0;
-    QgsRasterBlock* block = 0;
+    QgsRasterBlock *block = nullptr;
     mProcess->setReadChannel( QProcess::StandardOutput );
     mProgress->setRange( 0, mYSize - 1 );
     while ( iter.readNextRasterPart( band, iterCols, iterRows, &block, iterLeft, iterTop ) )
@@ -363,7 +359,7 @@ bool QgsGrassRasterImport::import()
 
         if ( !block->convert( qgis_out_type ) )
         {
-          setError( tr( "Cannot convert block (%1) to data type %2" ).arg( block->toString() ).arg( qgis_out_type ) );
+          setError( tr( "Cannot convert block (%1) to data type %2" ).arg( block->toString() ).arg( qgsEnumValueToKey<Qgis::DataType>( qgis_out_type ) ) );
           delete block;
           return false;
         }
@@ -377,19 +373,19 @@ bool QgsGrassRasterImport::import()
         {
           switch ( qgis_out_type )
           {
-            case QGis::Int32:
+            case Qgis::DataType::Int32:
               noDataValue = -2147483648.0;
               break;
-            case QGis::Float32:
+            case Qgis::DataType::Float32:
               noDataValue = std::numeric_limits<float>::max() * -1.0;
               break;
-            case QGis::Float64:
+            case Qgis::DataType::Float64:
               noDataValue = std::numeric_limits<double>::max() * -1.0;
               break;
             default: // should not happen
               noDataValue = std::numeric_limits<double>::max() * -1.0;
           }
-          for ( qgssize i = 0; i < ( qgssize )block->width()*block->height(); i++ )
+          for ( qgssize i = 0; i < ( qgssize ) block->width() * block->height(); i++ )
           {
             if ( block->isNoData( i ) )
             {
@@ -398,7 +394,7 @@ bool QgsGrassRasterImport::import()
           }
         }
 
-        char * data = block->bits( row, 0 );
+        char *data = block->bits( row, 0 );
         int size = iterCols * block->dataTypeSize();
         QByteArray byteArray = QByteArray::fromRawData( data, size ); // does not copy data and does not take ownership
         if ( isCanceled() )
@@ -411,7 +407,7 @@ bool QgsGrassRasterImport::import()
 
         outStream << byteArray;
 
-        // Without waitForBytesWritten() it does not finish ok on Windows (process timeout)
+        // Without waitForBytesWritten() it does not finish OK on Windows (process timeout)
         mProcess->waitForBytesWritten( -1 );
 
 #ifndef Q_OS_WIN
@@ -435,19 +431,22 @@ bool QgsGrassRasterImport::import()
     // TODO: best timeout?
     mProcess->waitForFinished( 30000 );
 
-    QString stdoutString = mProcess->readAllStandardOutput().data();
-    QString stderrString = mProcess->readAllStandardError().data();
+    QString stderrString = mProcess->readAllStandardError().constData();
 
-    QString processResult = QString( "exitStatus=%1, exitCode=%2, error=%3, errorString=%4 stdout=%5, stderr=%6" )
-                            .arg( mProcess->exitStatus() ).arg( mProcess->exitCode() )
-                            .arg( mProcess->error() ).arg( mProcess->errorString(),
-                                                           stdoutString.replace( "\n", ", " ), stderrString.replace( "\n", ", " ) );
-    QgsDebugMsg( "processResult: " + processResult );
+#ifdef QGISDEBUG
+    QString stdoutString = mProcess->readAllStandardOutput().constData();
+    QString processResult = QStringLiteral( "exitStatus=%1, exitCode=%2, error=%3, errorString=%4 stdout=%5, stderr=%6" )
+                              .arg( mProcess->exitStatus() )
+                              .arg( mProcess->exitCode() )
+                              .arg( mProcess->error() )
+                              .arg( mProcess->errorString(), stdoutString.replace( QLatin1String( "\n" ), QLatin1String( ", " ) ), stderrString.replace( QLatin1String( "\n" ), QLatin1String( ", " ) ) );
+    QgsDebugMsgLevel( "processResult: " + processResult, 3 );
+#endif
 
     if ( mProcess->exitStatus() != QProcess::NormalExit )
     {
       setError( mProcess->errorString() );
-      mProcess = 0;
+      mProcess = nullptr;
       return false;
     }
 
@@ -455,14 +454,14 @@ bool QgsGrassRasterImport::import()
     {
       setError( stderrString );
       delete mProcess;
-      mProcess = 0;
+      mProcess = nullptr;
       return false;
     }
 
     delete mProcess;
-    mProcess = 0;
+    mProcess = nullptr;
   }
-  QgsDebugMsg( QString( "redBand = %1 greenBand = %2 blueBand = %3" ).arg( redBand ).arg( greenBand ).arg( blueBand ) );
+  QgsDebugMsgLevel( QString( "redBand = %1 greenBand = %2 blueBand = %3" ).arg( redBand ).arg( greenBand ).arg( blueBand ), 3 );
   if ( redBand > 0 && greenBand > 0 && blueBand > 0 )
   {
     // TODO: check if the group exists
@@ -473,18 +472,18 @@ bool QgsGrassRasterImport::import()
     {
       QgsGrass::setMapset( mGrassObject.gisdbase(), mGrassObject.location(), mGrassObject.mapset() );
       struct Ref ref;
-      I_get_group_ref( name.toUtf8().data(), &ref );
-      QString redName = name + QString( ".%1" ).arg( redBand );
-      QString greenName = name + QString( ".%1" ).arg( greenBand );
-      QString blueName = name + QString( ".%1" ).arg( blueBand );
-      I_add_file_to_group_ref( redName.toUtf8().data(), mGrassObject.mapset().toUtf8().data(), &ref );
-      I_add_file_to_group_ref( greenName.toUtf8().data(), mGrassObject.mapset().toUtf8().data(), &ref );
-      I_add_file_to_group_ref( blueName.toUtf8().data(), mGrassObject.mapset().toUtf8().data(), &ref );
-      I_put_group_ref( name.toUtf8().data(), &ref );
+      I_get_group_ref( name.toUtf8().constData(), &ref );
+      QString redName = name + QStringLiteral( ".%1" ).arg( redBand );
+      QString greenName = name + QStringLiteral( ".%1" ).arg( greenBand );
+      QString blueName = name + QStringLiteral( ".%1" ).arg( blueBand );
+      I_add_file_to_group_ref( redName.toUtf8().constData(), mGrassObject.mapset().toUtf8().constData(), &ref );
+      I_add_file_to_group_ref( greenName.toUtf8().constData(), mGrassObject.mapset().toUtf8().constData(), &ref );
+      I_add_file_to_group_ref( blueName.toUtf8().constData(), mGrassObject.mapset().toUtf8().constData(), &ref );
+      I_put_group_ref( name.toUtf8().constData(), &ref );
     }
-    G_CATCH( QgsGrass::Exception &e )
+    G_CATCH( QgsGrass::Exception & e )
     {
-      QgsDebugMsg( QString( "Cannot create group: %1" ).arg( e.what() ) );
+      QgsDebugError( QString( "Cannot create group: %1" ).arg( e.what() ) );
     }
   }
   return true;
@@ -494,12 +493,12 @@ QString QgsGrassRasterImport::srcDescription() const
 {
   if ( !mPipe || !mPipe->provider() )
   {
-    return "";
+    return QString();
   }
   return mPipe->provider()->dataSourceUri();
 }
 
-QStringList QgsGrassRasterImport::extensions( QgsRasterDataProvider* provider )
+QStringList QgsGrassRasterImport::extensions( QgsRasterDataProvider *provider )
 {
   QStringList list;
   if ( provider && provider->bandCount() > 1 )
@@ -508,7 +507,7 @@ QStringList QgsGrassRasterImport::extensions( QgsRasterDataProvider* provider )
     list.reserve( bands );
     for ( int band = 1; band <= bands; ++band )
     {
-      list << QString( ".%1" ).arg( band );
+      list << QStringLiteral( ".%1" ).arg( band );
     }
   }
   return list;
@@ -519,7 +518,7 @@ QStringList QgsGrassRasterImport::names() const
   QStringList list;
   if ( mPipe && mPipe->provider() )
   {
-    Q_FOREACH ( const QString& ext, extensions( mPipe->provider() ) )
+    for ( const QString &ext : extensions( mPipe->provider() ) )
     {
       list << mGrassObject.name() + ext;
     }
@@ -532,9 +531,9 @@ QStringList QgsGrassRasterImport::names() const
 }
 
 //------------------------------ QgsGrassVectorImport ------------------------------------
-QgsGrassVectorImport::QgsGrassVectorImport( QgsVectorDataProvider* provider, const QgsGrassObject& grassObject )
-    : QgsGrassImport( grassObject )
-    , mProvider( provider )
+QgsGrassVectorImport::QgsGrassVectorImport( QgsVectorDataProvider *provider, const QgsGrassObject &grassObject )
+  : QgsGrassImport( grassObject )
+  , mProvider( provider )
 {
 }
 
@@ -542,7 +541,7 @@ QgsGrassVectorImport::~QgsGrassVectorImport()
 {
   if ( mFutureWatcher && !mFutureWatcher->isFinished() )
   {
-    QgsDebugMsg( "mFutureWatcher not finished -> waitForFinished()" );
+    QgsDebugMsgLevel( "mFutureWatcher not finished -> waitForFinished()", 3 );
     mFutureWatcher->waitForFinished();
   }
   delete mProvider;
@@ -550,29 +549,28 @@ QgsGrassVectorImport::~QgsGrassVectorImport()
 
 bool QgsGrassVectorImport::import()
 {
-
   if ( !mProvider )
   {
-    setError( "Provider is null." );
+    setError( QStringLiteral( "Provider is null." ) );
     return false;
   }
 
   if ( !mProvider->isValid() )
   {
-    setError( "Provider is not valid." );
+    setError( QStringLiteral( "Provider is not valid." ) );
     return false;
   }
 
   QgsCoordinateReferenceSystem providerCrs = mProvider->crs();
   QgsCoordinateReferenceSystem mapsetCrs = QgsGrass::crsDirect( mGrassObject.gisdbase(), mGrassObject.location() );
-  QgsDebugMsg( "providerCrs = " + providerCrs.toWkt() );
-  QgsDebugMsg( "mapsetCrs = " + mapsetCrs.toWkt() );
+  QgsDebugMsgLevel( "providerCrs = " + providerCrs.toWkt(), 3 );
+  QgsDebugMsgLevel( "mapsetCrs = " + mapsetCrs.toWkt(), 3 );
   QgsCoordinateTransform coordinateTransform;
   bool doTransform = false;
   if ( providerCrs.isValid() && mapsetCrs.isValid() && providerCrs != mapsetCrs )
   {
     coordinateTransform.setSourceCrs( providerCrs );
-    coordinateTransform.setDestCRS( mapsetCrs );
+    coordinateTransform.setDestinationCrs( mapsetCrs );
     doTransform = true;
   }
 
@@ -595,9 +593,9 @@ bool QgsGrassVectorImport::import()
   QDataStream outStream( mProcess );
   mProcess->setReadChannel( QProcess::StandardOutput );
 
-  QGis::WkbType wkbType = mProvider->geometryType();
-  bool isPolygon = QGis::singleType( QGis::flatType( wkbType ) ) == QGis::WKBPolygon;
-  outStream << ( qint32 )wkbType;
+  Qgis::WkbType wkbType = mProvider->wkbType();
+  bool isPolygon = QgsWkbTypes::singleType( QgsWkbTypes::flatType( wkbType ) ) == Qgis::WkbType::Polygon;
+  outStream << ( qint32 ) wkbType;
 
   outStream << mProvider->fields();
 
@@ -626,7 +624,7 @@ bool QgsGrassVectorImport::import()
       iterator.rewind();
     }
 #endif
-    QgsDebugMsg( "send features" );
+    QgsDebugMsgLevel( "send features", 3 );
     // Better to get real progress from module (without buffer)
 #if 0
     mProgress->setRange( 1, featureCount );
@@ -638,9 +636,11 @@ bool QgsGrassVectorImport::import()
       {
         continue;
       }
-      if ( doTransform && feature.geometry() )
+      if ( doTransform && feature.hasGeometry() )
       {
-        feature.geometry()->transform( coordinateTransform );
+        QgsGeometry g = feature.geometry();
+        g.transform( coordinateTransform );
+        feature.setGeometry( g );
       }
       if ( isCanceled() )
       {
@@ -650,14 +650,14 @@ bool QgsGrassVectorImport::import()
       outStream << false; // not canceled
       outStream << feature;
 
-      // Without waitForBytesWritten() it does not finish ok on Windows (data lost)
+      // Without waitForBytesWritten() it does not finish OK on Windows (data lost)
       mProcess->waitForBytesWritten( -1 );
 
 #ifndef Q_OS_WIN
       // wait until the feature is written to allow quick cancel (don't send data to buffer)
 
       // Feedback disabled because it was sometimes hanging on Linux, for example, when importing polygons
-      // the features were written ok in the first run, but after cleaning of polygons, which takes some time
+      // the features were written OK in the first run, but after cleaning of polygons, which takes some time
       // it was hanging here for few seconds after each feature, but data did not arrive to the modulee anyway,
       // QFSFileEnginePrivate::nativeRead() was hanging on fgetc()
 
@@ -678,22 +678,22 @@ bool QgsGrassVectorImport::import()
       // get some feedback for large datasets
       if ( count % 10000 == 0 )
       {
-        QgsDebugMsg( QString( "%1 features written" ).arg( count ) );
+        QgsDebugMsgLevel( QString( "%1 features written" ).arg( count ), 3 );
       }
     }
 
     feature = QgsFeature(); // indicate end by invalid feature
-    outStream << false; // not canceled
+    outStream << false;     // not canceled
     outStream << feature;
 
     mProcess->waitForBytesWritten( -1 );
-    QgsDebugMsg( "features sent" );
+    QgsDebugMsgLevel( "features sent", 3 );
 #ifndef Q_OS_WIN
 #if 0
     mProcess->waitForReadyRead();
     bool result;
     outStream >> result;
-    QgsDebugMsg( "got feedback" );
+    QgsDebugMsgLevel( "got feedback", 3 );
 #endif
 #endif
     iterator.close();
@@ -702,29 +702,34 @@ bool QgsGrassVectorImport::import()
   // Close write channel before waiting for response to avoid stdin buffer problem on Windows
   mProcess->closeWriteChannel();
 
-  QgsDebugMsg( "waitForReadyRead" );
+  QgsDebugMsgLevel( "waitForReadyRead", 3 );
   bool result;
   mProcess->waitForReadyRead();
   outStream >> result;
-  QgsDebugMsg( QString( "result = %1" ).arg( result ) );
+  QgsDebugMsgLevel( QString( "result = %1" ).arg( result ), 3 );
 
-  QgsDebugMsg( "waitForFinished" );
+  QgsDebugMsgLevel( "waitForFinished", 3 );
   mProcess->waitForFinished( 30000 );
 
-  QString stdoutString = mProcess->readAllStandardOutput().data();
-  QString stderrString = mProcess->readAllStandardError().data();
+#ifdef QGISDEBUG
+  QString stdoutString = mProcess->readAllStandardOutput().constData();
+#endif
+  QString stderrString = mProcess->readAllStandardError().constData();
 
-  QString processResult = QString( "exitStatus=%1, exitCode=%2, error=%3, errorString=%4 stdout=%5, stderr=%6" )
-                          .arg( mProcess->exitStatus() ).arg( mProcess->exitCode() )
-                          .arg( mProcess->error() ).arg( mProcess->errorString(),
-                                                         stdoutString.replace( "\n", ", " ), stderrString.replace( "\n", ", " ) );
-  QgsDebugMsg( "processResult: " + processResult );
+#ifdef QGISDEBUG
+  QString processResult = QStringLiteral( "exitStatus=%1, exitCode=%2, error=%3, errorString=%4 stdout=%5, stderr=%6" )
+                            .arg( mProcess->exitStatus() )
+                            .arg( mProcess->exitCode() )
+                            .arg( mProcess->error() )
+                            .arg( mProcess->errorString(), stdoutString.replace( QLatin1String( "\n" ), QLatin1String( ", " ) ), stderrString.replace( QLatin1String( "\n" ), QLatin1String( ", " ) ) );
+  QgsDebugMsgLevel( "processResult: " + processResult, 3 );
+#endif
 
   if ( mProcess->exitStatus() != QProcess::NormalExit )
   {
     setError( mProcess->errorString() );
     delete mProcess;
-    mProcess = 0;
+    mProcess = nullptr;
     return false;
   }
 
@@ -732,12 +737,12 @@ bool QgsGrassVectorImport::import()
   {
     setError( stderrString );
     delete mProcess;
-    mProcess = 0;
+    mProcess = nullptr;
     return false;
   }
 
   delete mProcess;
-  mProcess = 0;
+  mProcess = nullptr;
   return true;
 }
 
@@ -745,25 +750,20 @@ QString QgsGrassVectorImport::srcDescription() const
 {
   if ( !mProvider )
   {
-    return "";
+    return QString();
   }
   return mProvider->dataSourceUri();
 }
 
 //------------------------------ QgsGrassCopy ------------------------------------
-QgsGrassCopy::QgsGrassCopy( const QgsGrassObject& srcObject, const QgsGrassObject& destObject )
-    : QgsGrassImport( destObject )
-    , mSrcObject( srcObject )
-{
-}
-
-QgsGrassCopy::~QgsGrassCopy()
+QgsGrassCopy::QgsGrassCopy( const QgsGrassObject &srcObject, const QgsGrassObject &destObject )
+  : QgsGrassImport( destObject )
+  , mSrcObject( srcObject )
 {
 }
 
 bool QgsGrassCopy::import()
 {
-
   try
   {
     QgsGrass::copyObject( mSrcObject, mGrassObject );
@@ -784,25 +784,20 @@ QString QgsGrassCopy::srcDescription() const
 }
 
 //------------------------------ QgsGrassExternal ------------------------------------
-QgsGrassExternal::QgsGrassExternal( const QString& gdalSource, const QgsGrassObject& destObject )
-    : QgsGrassImport( destObject )
-    , mSource( gdalSource )
-{
-}
-
-QgsGrassExternal::~QgsGrassExternal()
+QgsGrassExternal::QgsGrassExternal( const QString &gdalSource, const QgsGrassObject &destObject )
+  : QgsGrassImport( destObject )
+  , mSource( gdalSource )
 {
 }
 
 bool QgsGrassExternal::import()
 {
-
   try
   {
     QString cmd = QgsGrass::gisbase() + "/bin/r.external";
     QStringList arguments;
 
-    if ( QFileInfo( mSource ).exists() )
+    if ( QFileInfo::exists( mSource ) )
     {
       arguments << "input=" + mSource;
     }
