@@ -14,29 +14,30 @@
  ***************************************************************************/
 
 
-#include <QtTest/QtTest>
+#include "qgstest.h"
 
 #include <editorwidgets/qgsspinbox.h>
+#include <QSignalSpy>
 
-class TestQgsSpinBox: public QObject
+class TestQgsSpinBox : public QObject
 {
     Q_OBJECT
   private slots:
-    void initTestCase(); // will be called before the first testfunction is executed.
+    void initTestCase();    // will be called before the first testfunction is executed.
     void cleanupTestCase(); // will be called after the last testfunction was executed.
-    void init(); // will be called before each testfunction is executed.
-    void cleanup(); // will be called after every testfunction.
+    void init();            // will be called before each testfunction is executed.
+    void cleanup();         // will be called after every testfunction.
 
     void clear();
     void expression();
+    void step();
+    void editingTimeout();
 
   private:
-
 };
 
 void TestQgsSpinBox::initTestCase()
 {
-
 }
 
 void TestQgsSpinBox::cleanupTestCase()
@@ -53,7 +54,7 @@ void TestQgsSpinBox::cleanup()
 
 void TestQgsSpinBox::clear()
 {
-  QgsSpinBox* spinBox = new QgsSpinBox();
+  QgsSpinBox *spinBox = new QgsSpinBox();
   spinBox->setMaximum( 10 );
   spinBox->setMinimum( 1 );
   spinBox->setValue( 5 );
@@ -74,7 +75,7 @@ void TestQgsSpinBox::clear()
 
 void TestQgsSpinBox::expression()
 {
-  QgsSpinBox* spinBox = new QgsSpinBox();
+  QgsSpinBox *spinBox = new QgsSpinBox();
   spinBox->setMinimum( -10 );
   spinBox->setMaximum( 10 );
   spinBox->setValue( 1 );
@@ -95,7 +96,7 @@ void TestQgsSpinBox::expression()
   QCOMPARE( spinBox->valueFromText( QString( "5/" ) ), 4 ); //invalid expression should reset to previous value
 
   //suffix tests
-  spinBox->setSuffix( QString( "mm" ) );
+  spinBox->setSuffix( QStringLiteral( "mm" ) );
   spinBox->setExpressionsEnabled( false );
   QCOMPARE( spinBox->valueFromText( QString( "5mm" ) ), 5 );
   QCOMPARE( spinBox->valueFromText( QString( "5+2mm" ) ), -10 );
@@ -112,7 +113,7 @@ void TestQgsSpinBox::expression()
 
   //prefix tests
   spinBox->setSuffix( QString() );
-  spinBox->setPrefix( QString( "mm" ) );
+  spinBox->setPrefix( QStringLiteral( "mm" ) );
   spinBox->setExpressionsEnabled( false );
   QCOMPARE( spinBox->valueFromText( QString( "mm5" ) ), 5 );
   QCOMPARE( spinBox->valueFromText( QString( "mm5+2" ) ), -10 );
@@ -128,8 +129,8 @@ void TestQgsSpinBox::expression()
   QCOMPARE( spinBox->valueFromText( QString( "mm5/" ) ), 4 ); //invalid expression should reset to previous value
 
   //both suffix and prefix
-  spinBox->setSuffix( QString( "ll" ) );
-  spinBox->setPrefix( QString( "mm" ) );
+  spinBox->setSuffix( QStringLiteral( "ll" ) );
+  spinBox->setPrefix( QStringLiteral( "mm" ) );
   spinBox->setExpressionsEnabled( true );
   QCOMPARE( spinBox->valueFromText( QString( "mm 5 ll" ) ), 5 );
   QCOMPARE( spinBox->valueFromText( QString( "mm 5+2 ll" ) ), 7 );
@@ -144,5 +145,101 @@ void TestQgsSpinBox::expression()
   delete spinBox;
 }
 
-QTEST_MAIN( TestQgsSpinBox )
+void TestQgsSpinBox::step()
+{
+  // test step logic
+
+  QgsSpinBox spin;
+  spin.setMinimum( -1000 );
+  spin.setMaximum( 1000 );
+  spin.setSingleStep( 1 );
+
+  // no clear value
+  spin.setValue( 0 );
+  spin.stepBy( 1 );
+  QCOMPARE( spin.value(), 1 );
+  spin.stepBy( -1 );
+  QCOMPARE( spin.value(), 0 );
+  spin.stepBy( -1 );
+  QCOMPARE( spin.value(), -1 );
+
+  // with clear value
+  spin.setClearValue( -1000, QStringLiteral( "NULL" ) );
+  spin.setValue( 0 );
+  spin.stepBy( 1 );
+  QCOMPARE( spin.value(), 1 );
+  spin.stepBy( -1 );
+  QCOMPARE( spin.value(), 0 );
+  spin.stepBy( -1 );
+  QCOMPARE( spin.value(), -1 );
+  spin.clear();
+  QCOMPARE( spin.value(), -1000 );
+  // when cleared, a step should NOT go to -999 (which is annoying for users), but rather pretend that the initial value was 0, not NULL
+  spin.stepBy( 1 );
+  QCOMPARE( spin.value(), 1 );
+  spin.clear();
+  QCOMPARE( spin.value(), -1000 );
+  spin.stepBy( -1 );
+  QCOMPARE( spin.value(), -1 );
+
+  // with clear value, but no special value text. In this case we should NOT reset to 0 when incrementing up from the clear value
+  spin.setSpecialValueText( QString() );
+  spin.setClearValue( -1000 );
+  spin.setValue( 0 );
+  spin.stepBy( 1 );
+  QCOMPARE( spin.value(), 1 );
+  spin.stepBy( -1 );
+  QCOMPARE( spin.value(), 0 );
+  spin.stepBy( -1 );
+  QCOMPARE( spin.value(), -1 );
+  spin.clear();
+  QCOMPARE( spin.value(), -1000 );
+  spin.stepBy( 1 );
+  QCOMPARE( spin.value(), -999 );
+  spin.clear();
+  QCOMPARE( spin.value(), -1000 );
+  spin.stepBy( -1 );
+  QCOMPARE( spin.value(), -1000 );
+}
+
+void TestQgsSpinBox::editingTimeout()
+{
+  QgsSpinBox spin;
+  spin.setMinimum( -1000 );
+  spin.setMaximum( 1000 );
+  spin.setSingleStep( 1 );
+  spin.setFocus();
+  QCOMPARE( spin.editingTimeoutInterval(), 1000 );
+  spin.setEditingTimeoutInterval( 300 );
+  QCOMPARE( spin.editingTimeoutInterval(), 300 );
+
+  QSignalSpy spy( &spin, &QgsSpinBox::editingTimeout );
+  spin.selectAll();
+  QTest::keyClicks( &spin, QStringLiteral( "3" ) );
+  QTest::qWait( 100 );
+  // too short, should not be signal
+  QCOMPARE( spy.count(), 0 );
+  QTest::qWait( 400 );
+  // long enough, signal should have been emitted
+  QCOMPARE( spy.count(), 1 );
+  QCOMPARE( spy.at( 0 ).at( 0 ).toInt(), 3 );
+
+  QTest::keyClicks( &spin, QStringLiteral( "2" ) );
+  QCOMPARE( spy.count(), 1 );
+  QTest::qWait( 400 );
+  // long enough, signal should have been emitted
+  QCOMPARE( spy.count(), 2 );
+  QCOMPARE( spy.at( 1 ).at( 0 ).toInt(), 32 );
+
+  // no signal if value not changed
+  QTest::keyClicks( &spin, QStringLiteral( "4" ) );
+  QTest::qWait( 100 );
+  QCOMPARE( spy.count(), 2 );
+  QTest::keyPress( &spin, Qt::Key_Backspace );
+  QTest::qWait( 400 );
+  // no signal, value did not change
+  QCOMPARE( spy.count(), 2 );
+}
+
+QGSTEST_MAIN( TestQgsSpinBox )
 #include "testqgsspinbox.moc"
